program cdf_matrix_2d

  use sep
  use stats_mod

  implicit none

  integer                               :: ntau,nv,ncmp,i,j,k,itau,icmp,ind1,ind2,pp
  real                                  :: otau,dtau,ov,dv,ocmp,dcmp,mean_mean,mean_var,test
  real,dimension(:,:,:),allocatable     :: detrended_semblance,coord_detrend,cdf
  integer,dimension(:,:),allocatable    :: length
  real,dimension(:,:),allocatable       :: mean,var,total_weight

  call sep_init()

  call from_history("n1",ntau)
  call from_history("o1",otau)
  call from_history("d1",dtau)
  call from_history("n2",nv)
  call from_history("o2",ov)
  call from_history("d2",dv)
  call from_history("n3",ncmp)
  call from_history("o3",ocmp)
  call from_history("d3",dcmp)
 
  call from_param("itau",itau)
  call from_param("icmp",icmp)

  allocate(detrended_semblance(ntau,nv,ncmp))
  allocate(coord_detrend(ntau,3,ncmp))
  allocate(length(ntau,ncmp))
  allocate(mean(ntau,ncmp))
  allocate(var(ntau,ncmp))
  allocate(total_weight(ntau,ncmp))

  call sep_read(detrended_semblance)
  call sep_read(coord_detrend,"coord")

  do j=1,ncmp
    do i=1,ntau
      length(i,j) = coord_detrend(i,2,j)
    end do
  end do

  mean = 0
  var = 0
  total_weight = 0

  do k=1,ncmp
    do i=1,ntau
      total_weight(i,k) = sum(detrended_semblance(i,1:length(i,k),k))
      do j=1,length(i,k)
        mean(i,k) = mean(i,k) + detrended_semblance(i,j,k) * (coord_detrend(i,1,k)+(j-1)*coord_detrend(i,3,k))
      end do
      mean(i,k) = mean(i,k)/total_weight(i,k)
    end do
  end do

  do k=1,ncmp
    do i=1,ntau
      total_weight(i,k) = sum(detrended_semblance(i,1:length(i,k),k))
      do j=1,length(i,k)
        var(i,k) = var(i,k) + detrended_semblance(i,j,k) * (coord_detrend(i,1,k)+(j-1)*coord_detrend(i,3,k) - mean(i,k))**2
      end do
      var(i,k) = var(i,k) / total_weight(i,k)
    end do
  end do

  mean_mean = 0
  mean_var  = 0

  do k=1,ncmp  
    do i=1,ntau
      mean_mean = mean_mean + mean(i,k)
      mean_var  = mean_var  +  var(i,k)
    end do
  end do
  mean_mean = mean_mean / (ntau * ncmp)
  mean_var = mean_var / (ntau * ncmp)

  write(0,*) "mean of mean",mean_mean
  write(0,*) "mean of var",mean_var

  allocate(cdf(ntau,nv,ncmp))
  write(0,*) "nv",nv

  cdf = 0

  do k=1,ncmp
    do i=1,ntau
      cdf(i,1:length(i,k),k) = raw_cdf(detrended_semblance(i,1:length(i,k),k),coord_detrend(i,1,k),coord_detrend(i,3,k),coord_detrend(i,1,k),coord_detrend(i,1,k) + (length(i,k)-1)*coord_detrend(i,3,k)) 
    end do
  end do
  ind1 = 30
  ind2 = 100
  !pp = size(detrended_semblance(ind,1:length(ind)))
  write(0,*) "pp",length(ind1,ind2)
  
  test = mc_gaussian(cdf(ind1,1:length(ind1,ind2),ind2),coord_detrend(ind1,1,ind2),coord_detrend(ind1,3,ind2),length(ind1,ind2))
  write(0,*) "test",test

  call sep_write(cdf)
  call to_history("n1",ntau)
  call to_history("o1",otau)
  call to_history("d1",dtau)
  call to_history("n2",nv)
  call to_history("o2",1)
  call to_history("d2",1)
  call to_history("n3",ncmp)
  call to_history("o3",ocmp)
  call to_history("d3",dcmp)

  call sep_write(cdf(itau,:,icmp),"plot_cdf")
  call to_history("n1",length(itau,icmp),"plot_cdf")
  call to_history("o1",coord_detrend(itau,1,icmp),"plot_cdf")
  call to_history("d1",coord_detrend(itau,3,icmp),"plot_cdf")
  call to_history("n2",1,"plot_cdf")
  call to_history("o2",0,"plot_cdf")
  call to_history("d2",0,"plot_cdf")

  call sep_write(detrended_semblance(itau,:,icmp),"plot_density")
  call to_history("n1",length(itau,icmp),"plot_density")
  call to_history("o1",coord_detrend(itau,1,icmp),"plot_density")
  call to_history("d1",coord_detrend(itau,3,icmp),"plot_density")
  call to_history("n2",1,"plot_density")
  call to_history("o2",0,"plot_density")
  call to_history("d2",0,"plot_density")

  call sep_write(mean,"mean")
  call to_history("n1",ntau,"mean")
  call to_history("o1",otau,"mean")
  call to_history("d1",dtau,"mean")
  call to_history("n2",1,"mean")
  call to_history("o2",0,"mean")
  call to_history("d2",0,"mean")

  call sep_write(var,"var")
  call to_history("n1",ntau,"var")
  call to_history("o1",otau,"var")
  call to_history("d1",dtau,"var")
  call to_history("n2",1,"var")
  call to_history("o2",0,"var")
  call to_history("d2",0,"var")

  call sep_close()    

end program
