module covariance_class
! this module creates a variogram 'class' based on an exponential model for the correlation
  implicit none
  public :: covariance

  type covariance
    real                     :: nugget,range,sill,alpha
    contains
      procedure              :: get_covariance_value
  end type
  
  interface covariance
    module procedure covariance_constructor
  end interface
  
  contains
  
  function covariance_constructor(input_nugget,input_range,input_sill,input_alpha) result(new_covariance)
    type(covariance)    :: new_covariance
    real                :: input_nugget,input_range,input_sill

    new_covariance%nugget = input_nugget
    new_covariance%range  = input_range
    new_covariance%sill   = input_sill
    new_covariance%alpha  = input_alpha
  end function

  function get_covariance_value(this,distance) result(covariance_value)
    class(covariance)       :: this
    real                    :: distance,covariance_value,coeff

    coeff = this%sill - this%nugget
    if (distance == 0.) then
      covariance_value = this%sill
    else 
      covariance_value = (this%sill - this%nugget)*exp(-this%alpha/this%range*abs(distance))
    end if
    
  end function
  
end module
