module covariance_mod

  implicit none
 
  contains

  subroutine covariance_function(data,direction,nb_pairs_vector,lag_to_distance,covariance)
    real,dimension(:,:)                :: data
    integer,dimension(:)               :: direction ! first component is z direction, second is x direction
    real,dimension(:)                  :: covariance,lag_to_distance
    integer                            :: n1,n2,lag,max_lag,i,j,nb_pairs,i_new,j_new
    integer,dimension(:,:),allocatable :: flag1,flag2    
    real                               :: average1,average2,dist_lag
    integer,dimension(:)               :: nb_pairs_vector ! vector containing the number of pairs used for the covariance calculation, for each alg (first component is for lag=0)

    n1 = size(data,1)
    n2 = size(data,2)

    ! flag1 is a matrix that tells you whether the point has been use in one of the pairs for the covariance calculation
    ! flag2 is a similar matrix that tells you if the point has been used in one of the pairs 

    allocate(flag1(n1,n2))
    allocate(flag2(n1,n2))

    !max lag is the maximum lag (as a multiple of the direction vector) 
    max_lag = size(covariance,1)

    !allocate(covariance(max_lag))
    !allocate(nb_pairs_vector(max_lag))
    !allocate(lag_to_distance(max_lag)) !translates the "lags" in terms of distance in nb of samples

    lag_to_distance = 0.

    ! converting one lag in terms of nb of samples
    dist_lag = direction(1)*direction(1) + direction(2)*direction(2)
    !write(0,*) "dist_lag", dist_lag
    dist_lag = sqrt(dist_lag)
    !write(0,*) "dist_lag", dist_lag
    do i=1,max_lag
      lag_to_distance(i) = dist_lag * (i-1)
    end do 

    nb_pairs_vector = 0 ! initializing the nb of pairs to zero for all lags 

    do lag=1,max_lag ! we pick a direction vector and we multiply the vector by distance 

      ! Finding the pairs of points that will contribute to the covariance function
      flag1 = 0 ! reset flag1 matrix to zero because we change the lag
      flag2 = 0 ! reset flag2 to zero
      do i=1,n1
        i_new = i + direction(1) * (lag-1)  
        if ((i_new .le. n1) .and. (i_new .ge. 1))  then 
          do j=1,n2
            j_new = j + direction(2) * (lag-1)
            if ((j_new .le. n2) .and. (j_new .ge. 1)) then 
              nb_pairs_vector(lag) = nb_pairs_vector(lag) + 1
              flag1(i,j)=1
              flag2(i_new,j_new)=1
              covariance(lag) = covariance(lag) + data(i,j) * data(i_new,j_new)
            end if
          end do
        end if
      end do  

      if (nb_pairs_vector(lag) .gt. 0) then
        covariance(lag) = 1./nb_pairs_vector(lag) * covariance(lag) ! calculate the first term in eq. (4.2) of book (p.59)
        average1 = 0. ! calculating the second term in eq. (4.2) of book (p.59)
        average2 = 0.
        do i=1,n1
          do j=1,n2
            if (flag1(i,j) .eq. 1) then
              average1 = average1 + data(i,j)
            end if
            if (flag2(i,j) .eq. 1) then
              average2 = average2 + data(i,j)
            end if
          end do
        end do
        average1 = average1/nb_pairs_vector(lag)
        average2 = average2/nb_pairs_vector(lag)
        covariance(lag) = covariance(lag) - average1 * average2
      end if
    end do

  end subroutine

end module

