program generate_velocity_model

! this program works only for 2D data sets
! it takes into input a .H file, which is a 3D matrix M(#cmp,ntau,nv). 
! for each #CMP i, M(i,:,:) is a 2D matrix respresentin a semblance plot for CMP #i
! the program generates a random 2D velocity field in an output.H file

  use sep
  use stats_functions_mod
  use geostats_functions_mod
  
  implicit none
  
  integer                            :: ntau,nv,nt,nx,ncmp,nb_pseudo_samples,min_nb_krigging  
  real                               :: otau,dtau,ov,dv,ot,dt,ox,dx,ocmp,dcmp,min_vel,max_vel
  real,dimension(:,:),allocatable    :: cdf,velocity_model
  logical                            :: test_generation
  integer,dimension(:,:),allocatable :: flag_matrix
  real,dimension(:,:,:),allocatable  :: semblance
  real                               :: harvest
  integer                            :: it,i,x_coord
 
  call sep_init() ! call this subroutine in order to be able to use the sep library

  ! reading parameters for the output file: the velocity model (nt, nx, etc.)
  call from_param("nt",nt)
  call from_param("ot",ot)
  call from_param("dt",dt)
  call from_param("nx",nx)
  call from_param("ox",ox)
  call from_param("dx",dx)
  call from_param("nb_pseudo_samples",nb_pseudo_samples) ! when we start constructing the velocity model, we start (for each depth) by generating "random" samples (by some kind of Monte Carlo). Then we do krigging to obtain the rest of the values. This parameter determines the number of points (at a given depth) that should be chosen randomly.
  call from_param("min_velocity",min_vel) ! minimum velocity allowed
  call from_param("max_velocity",max_vel) ! maximum velocity allowed
  call from_param("min_nb_points_krigging",min_nb_krigging) ! minimum number of points needed to do krigging
  call from_param("scanning_distance",scan_dist !for each point in the subsurface, we scan if there are already assigned velocity values for points within the scanning distance. If we don't find enough, we generate randomly

  ! importing parameters from the semblance file
  call from_history("n1", ntau) 
  call from_history("o1", otau)
  call from_history("d1", dtau)
  call from_history("n2", nv) 
  call from_history("o2", ov)
  call from_history("d2", dv)
  call from_history("n3",ncmp)
  call from_history("o3",ocmp)
  call from_history("d3",dcmp)

  ! allocating and storing the data into the local variable semblance
  allocate(semblance(ntau,nv,ncmp))
  call sep_read(semblance)
  
  ! allocating and initializing the flag matrix to zero. Flag is 0 when the point hasn't been assigned a velocity value yet.
  allocate(flag_matrix(nt,nx))
  flag_matrix=0

  ! allocating the velocity model (output)
  allocate(velocity_model(nt,nx))

  ! starting to generate random velocity values for the velocity model
  do it=1,nt
    do i=1,nb_pseudo_samples    
      test_generation = .false.
      while (test_generation .eqv. false)
        call random_number(harvest)
        x_coord=ox+harvest*(nx-1)*dx
        ix=1.5+(x_coord-ox)/dx   
        if (flag_matrix(it,ix) == 0) then
          velocity(it,ix) = generate_velocity_pick_gaussian(semblance(ix,:,:),it,ot,ov,dt,dv,min_vel,max_vel)
          flag_matrix(it,ix)=1
          test_generation= .true.
        end if
      end while 
    end do
  end do      
  
  ! applying krigging the points to which I haven't assigned a velocity value yet
  do it=1,nt
    do ix=1,nx
      if (flag_matrix(it,ix) == 0) then
        scan
        velocity(it,ix)= apply_krigging()
      end if
    end do
  end do

  call sep_close()
end program

