! Kirchhoff migration and diffraction. (greased lightning)
module Kirchhoff

  implicit none
  integer, private                         :: nt, nx
  real, private                            :: amax, apermax, t0, dt, dx
  logical, private                         :: isamax
  real, dimension(:), allocatable, private :: velhalf

  contains

  subroutine kirchfast_init(velhalf_in, t0_in, dt_in, dx_in, nt_in, nx_in, isamax_in)
    integer                         :: nt_in, nx_in
    real                            :: t0_in, dt_in, dx_in
    logical                         :: isamax_in
    real, dimension(:), allocatable :: velhalf_in
    nt = nt_in
    nx = nx_in
    t0 = t0_in
    dt = dt_in
    dx = dx_in
    isamax = isamax_in
    allocate(velhalf(nt))
    velhalf = 0.
    velhalf = velhalf_in
  end subroutine

  subroutine update_limit(amax_in, apermax_in)
    real                            :: amax_in, apermax_in
    amax = amax_in
    apermax = apermax_in
  end subroutine

  subroutine kirchfast(adj, add, modl, data)  
    logical, intent(in)   :: adj, add
    real, dimension(:,:)  :: modl, data
    integer               :: ix, iz, it, ih, xstart, xend
    real                  :: amp, t, z, h , amax_rad !, weighting
    real                  :: pi=acos(-1.0)

    if(.not. add) then
      if(adj) then 
        modl = 0.
      else 		    
        data = 0.
      endif
    endif
    
    amax_rad=amax*pi/180  
! v(z)=1.5+alpha*z
    do ih=-nx,nx
      h = dx * ih ! h = offset
      do iz= 2, nt
        z = t0 + dt * (iz-1) ! z = travel-time depth
        t = sqrt( z**2 + (h/velhalf(iz))**2 )
        
        if(t>z/cos(amax_rad)) then
          cycle
        endif
        
        !if (t<z/cos(amax_rad)) then
        !  weighting=1
        !else 
        !  weighting=(90-acos(z/t)*180/pi)/(90-amax)
        !  weighting = (sin(pi/2*weighting))**10
        !end if
        
        it = 1.5 + (t - t0) / dt
        if(it > nt) then
          exit
        endif
        
        amp = (z / t) * sqrt( nt*dt / t ) !* weighting
        xstart = 1-ih
        xstart = max0( 1, xstart)
        xend = nx-ih
        xend = min0( nx, xend)
        if(.not. adj) then
          do ix= xstart, xend
            data(it,ix+ih) = data(it,ix+ih) + modl(iz,ix)*amp
          enddo
        else
          do ix= xstart, xend
            modl(iz,ix) = modl(iz,ix ) + data(it,ix+ih)*amp
          enddo
        endif
      enddo
    enddo
  end subroutine

end module
