program kriging_program

  use sep
  use scan_mod
  use kriging_mod
  use stats_mod

  implicit none

  integer                              :: nz,nx,n,range,n_min,n_max,nb1,nb2,nb3,nb4,known_samp,nb_samp_krig,  init_known_samp
  real                                 :: oz,dz,ox,dx,z_coord,x_coord
  real,dimension(:,:),allocatable      :: sparse_data,temp,stand_dev_mat
  integer,dimension(:,:),allocatable   :: flag,flag_krig,flag_mc,flag_start,flag_seq_sim
  real,dimension(:,:),allocatable      :: rflag,rflag_krig,rflag_mc,rflag_start,rflag_sum,rflag_seq_sim
  real                                 :: rand_nb
  logical                              :: stop_krig
  integer,dimension(2)                 :: position
  integer                              :: i,j,k,by_krig,by_mc,beginning,at_end,seq_sim,rand_int
  real                                 :: nugget,range_var,sill,alpha
  real,dimension(:),allocatable        :: samp_values,cdf,seq_cdf,ex_seq_cdf
  integer,dimension(:,:),allocatable   :: samp_position,samp1,samp2,samp3,samp4
  integer                              :: counter,niter,n_cdf,n_stop,seq_n_cdf,nb_stand_dev,by_seq_sim,p
  real                                 :: sigma,o_cdf,d_cdf,avg,power,error_var
  real                                 :: temp_krig_value,error_stand_dev,seq_o_cdf,seq_f_cdf,seq_d_cdf
  integer,dimension(1)                 :: seed
  real                                 :: ex_o_cdf,ex_d_cdf
  integer                              :: ex_n_cdf,rand_counter,index_rand,nb_random_calls,prior_rand
  real,dimension(:),allocatable        :: rand_vect
  real                                 :: t_start,t_end,value_for_cdf
  integer                              :: mean_sparse,cc
  real,dimension(:,:),allocatable      :: coord_detrend,cdf_mat
  integer                              :: ntau,nh
  real                                 :: otau,dtau,oh,dh
  integer,dimension(:),allocatable     :: length_cdf
  real,dimension(:),allocatable        :: temp_merde
  real,dimension(:),allocatable        :: mc_vect

  call cpu_time(t_start)
  call sep_init()

  call from_history("n1",nz) 
  call from_history("o1",oz)
  call from_history("d1",dz)
  call from_history("n2",nx) 
  call from_history("o2",ox)
  call from_history("d2",dx)

  !!!!!! parameters for scanning !!!!!!
  call from_param("n_max",n_max) 
  call from_param("n_min",n_min)
  call from_param("range",range)

  !!!!!! parameters for variogram !!!!!!
  call from_param("nugget",nugget)
  call from_param("range_variogram",range_var)
  call from_param("sill",sill)  !sill is the variance for our stationary model
  call from_param("alpha",alpha)
  call from_param("niter",niter)
  call from_param("avg",avg) ! import the mean for our stationary model
  call from_param("power",power)

  !!!!!! parameters for CDF !!!!!!
  call from_param("n_cdf",n_cdf)
  call from_param("o_cdf",o_cdf)
  call from_param("d_cdf",d_cdf)
 
  !!!!!!! parameter for Gaussian sequential simulation !!!!!!! 
  call from_param("seq_sim",seq_sim)
  call from_param("nb_stand_dev",nb_stand_dev) ! how far off the mean you want to reach values in the  sequential simulation. ex: 3 implies that we search [kriged value - 3sigma ; kriged_value + 3sigma] 
  call from_param("seq_n_cdf",seq_n_cdf)
  call from_param("rand_int",rand_int) ! seed parameter for the sequence of random number
  call from_param("nb_random_calls",nb_random_calls)
  call from_param("prior",prior_rand)

  write(0,*) "prior rand",prior_rand
  value_for_cdf = 0
  !!!!!!! random calls !!!!!!!
  rand_counter = 0
  if (prior_rand .eq. 1) then
    allocate(rand_vect(nb_random_calls))
    write(0,*) "just allocated"
    seed(1) = rand_int
    call random_seed(PUT=seed) 
    call random_number(rand_vect) !generates the big vector containing the random values
    write(0,*) "just gerenated"
    index_rand = 1
  else 
    write(0,*) "normal random calls"
    seed(1) = rand_int
    call random_seed(PUT=seed) 
    call random_number(rand_nb)
  end if

  p = 0
  call ordinary_kriging_init(nugget,range_var,sill,alpha,power,niter) !!!! initializing the ordinary kriging function

  !!!!!!!!!!!!!!!!!!!!!!!!!!! Generating the general cumulative density function (CDF) !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  sigma = sqrt(sill)
  allocate(cdf(n_cdf))
  cdf = gaussian_cdf(o_cdf,d_cdf,n_cdf,avg,sigma)

  allocate(sparse_data(nz,nx))
  allocate(flag(nz,nx))
  allocate(temp(nz,nx))

  allocate(stand_dev_mat(nz,nx))
  stand_dev_mat = 0.

  !!!!!!!! Importing the CDF from the semblance plots !!!!!!!
  !call from_param("ntau",ntau)
  !call from_param("otau",otau)
  !call from_param("dtau",dtau)
  !call from_param("nh",nh)
  !call from_param("oh",oh)
  !call from_param("dh",dh)

  allocate(cdf_mat(nz,nx))
  allocate(coord_detrend(nz,nx))
  allocate(length_cdf(nz))
  
  call sep_read(cdf_mat,"cdf_mat")
  call sep_read(coord_detrend,"coord_detrend")
  do i=1,nz
    length_cdf(i) = coord_detrend(i,2)
  end do

  call sep_read(sparse_data) 
  call sep_read(temp,"flag") ! flag_matrix.H is an auxiliary file
  flag = temp ! flag has to be of type integer, whereas temp is the array read using sep_read (so we need to convert it from real -> integer).
  deallocate(temp)

  allocate(flag_krig(nz,nx)) !flag_krig=1 when the value has been determined using kriging
  allocate(flag_mc(nz,nx)) ! flag_mc = 1 when the value has been generated usign MC
  allocate(flag_start(nz,nx)) ! flag_start=1 when the value was known from the start
  allocate(flag_seq_sim(nz,nx)) !flag for Gaussian sequential simulation
  allocate(rflag(nz,nx)) ! flag=1 when the point has a known value
  allocate(rflag_krig(nz,nx)) !flag_krig=1 when the value has been determined using kriging
  allocate(rflag_mc(nz,nx)) ! flag_mc = 1 when the value has been generated usign MC
  allocate(rflag_start(nz,nx)) ! flag_start=1 when the value was known from the start
  allocate(rflag_sum(nz,nx))
  allocate(rflag_seq_sim(nz,nx)) !flag for Gaussian sequential simulation

  !!!! calculating the mean of spare data
  mean_sparse = 0
  cc = 0
  do i=1,nz
    do j=1,nx
      if (flag(i,j) .eq. 1) then
        mean_sparse = mean_sparse + sparse_data(i,j)
        cc = cc + 1
      end if
    end do
  end do
  if (cc .ne.0) mean_sparse = mean_sparse/cc

  !!!!!!!!  saving the starting flag array in flag_start !!!!!!!! 
  flag_start = flag 
  flag_krig = 0
  flag_mc = 0
  flag_seq_sim = 0
 
  !!!!!!!! getting the original nb of known samples !!!!!!!!
  known_samp = 0
  do i=1,nz
    do j=1,nx
      known_samp = known_samp + flag(i,j)
    end do
  end do

  !!!!!!!!  starting sample generation process !!!!!!!!
  n = nz * nx
  counter = 0
  !init_known_samp = known_samp

  do while ((known_samp .lt. n)) ! full interpolation
  !do while ((counter .lt. 1)) !.and. (counter .lt. 1)) ! interpolate only for a limited number of points

!    write(0,*) "mod",mod(counter,100)

    if (mod(counter,10000) .eq. 0) then
      write(0,*) "nb",known_samp
      write(0,*) "remaining to interpolate", n - known_samp
      write(0,*) "already found",counter
    end if

!    call random_number(rand_nb)
!    rand_counter = rand_counter + 1

    if (prior_rand .eq. 1) then
      index_rand = index_rand + 1
      rand_nb = rand_vect(index_rand)
    else
      call random_number(rand_nb)
    end if

    x_coord = ox + (rand_nb)*(nx-1)*dx
    j = 1.5 + (x_coord-ox)/dx

!    call random_number(rand_nb)
!    rand_counter = rand_counter + 1

    if (prior_rand .eq. 1) then
      index_rand = index_rand + 1
      rand_nb = rand_vect(index_rand)
    else
      call random_number(rand_nb)
    end if

    z_coord = oz + (rand_nb)*(nz-1)*dz
    i = 1.5 + (z_coord-oz)/dz

    if (flag(i,j) .eq. 0) then
      counter = counter + 1

      position(1) = i ; position(2) = j     

      call scan2d(range,position,n_min,n_max,flag,stop_krig,samp_position,samp_values,sparse_data)
      if (stop_krig .eqv. .false.) then 

        if (seq_sim .eq. 0) then ! just do kriging, no SGS
          sparse_data(i,j) = ordinary_kriging(samp_position,samp_values,position,error_var) 
          !write(0,*) "kriged value",sparse_data(i,j)
          flag_krig(i,j) = 1
          !write(0,*) "finshed kriging, counter",counter

          deallocate(samp_position)
          deallocate(samp_values)

        else ! kriging with Gaussian sequential simulation (SGS)

          temp_krig_value = ordinary_kriging(samp_position,samp_values,position,error_stand_dev)

          seq_o_cdf = temp_krig_value - nb_stand_dev * error_stand_dev
          seq_f_cdf = temp_krig_value + nb_stand_dev * error_stand_dev
          seq_d_cdf = (seq_f_cdf - seq_o_cdf) / seq_n_cdf    
          stand_dev_mat(i,j) = error_stand_dev    

          allocate(seq_cdf(seq_n_cdf)) 
 
          seq_cdf = gaussian_cdf(seq_o_cdf,seq_d_cdf,seq_n_cdf,temp_krig_value,error_stand_dev)
          if (prior_rand .eq. 1) then
            sparse_data(i,j) = mc_gaussian(seq_cdf,seq_o_cdf,seq_d_cdf,seq_n_cdf,rand_vect,index_rand)
          else
            sparse_data(i,j) = mc_gaussian(seq_cdf,seq_o_cdf,seq_d_cdf,seq_n_cdf)
          end if
            
          flag_seq_sim(i,j) = 1

          p = p + 1
          if (p .eq. 1) then
            allocate(ex_seq_cdf(seq_n_cdf)) 
            ex_seq_cdf = seq_cdf
            ex_n_cdf = seq_n_cdf
            ex_o_cdf = seq_o_cdf
            ex_d_cdf = seq_d_cdf
            value_for_cdf = error_stand_dev
          end if 
    
          deallocate(samp_position)
          deallocate(samp_values)
          deallocate(seq_cdf)       
  
        end if

      else

        if (prior_rand .eq. 1) then
          !sparse_data(i,j) = mc_gaussian(cdf,o_cdf,d_cdf,n_cdf,rand_vect,index_rand)
          sparse_data(i,j) = mc_gaussian(cdf_mat(i,1:length_cdf(i)),coord_detrend(i,1),coord_detrend(i,3),length_cdf(i),rand_vect,index_rand)
        else 
          !sparse_data(i,j) = mc_gaussian(cdf,o_cdf,d_cdf,n_cdf)
          sparse_data(i,j) = mc_gaussian(cdf_mat(i,1:length_cdf(i)),coord_detrend(i,1),coord_detrend(i,3),length_cdf(i))
        end if
        flag_mc(i,j) = 1
        !write(0,*) "value generated by MC"

      end if

      flag(i,j) = 1
      known_samp = known_samp + 1

    end if
  end do

  ! checking seq cdf

  rflag_sum = 1.0 * flag_start + 1.0 * flag_krig 
  rflag = 1.0 * flag
  rflag_krig = 1.0 * flag_krig
  rflag_mc = 1.0 * flag_mc
  rflag_seq_sim = 1.0 * flag_seq_sim

  !!!!!! displaying the nb of points for each method

  by_krig=0;by_mc=0;at_end=0;beginning=0;by_seq_sim=0
  do i=1,nz
    do j=1,nx
      by_krig    = by_krig    + flag_krig(i,j)
      by_mc      = by_mc      + flag_mc(i,j)
      beginning  = beginning  + flag_start(i,j)
      at_end     = at_end     + flag(i,j)
      by_seq_sim = by_seq_sim + flag_seq_sim(i,j)
    end do

  end do
  write(0,*) "total points", n
  write(0,*) "krig:",by_krig
  write(0,*) "mc:",by_mc
  write(0,*) "beginning:",beginning
  write(0,*) "end:",at_end
  write(0,*) "sequential sim:",by_seq_sim
  write(0,*) "random count", rand_counter
  write(0,*) "variance for example", value_for_cdf


  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! output !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  call sep_write(sparse_data)
  call to_history("n1",nz) 
  call to_history("o1",oz)
  call to_history("d1",dz)
  call to_history("n2",nx) 
  call to_history("o2",ox)
  call to_history("d2",dx)

  if (seq_sim .eq. 0) then
    call sep_write(rflag_krig,"flag_krig")
    call to_history("n1",nz,"flag_krig") 
    call to_history("o1",oz,"flag_krig") 
    call to_history("d1",dz,"flag_krig") 
    call to_history("n2",nx,"flag_krig")  
    call to_history("o2",ox,"flag_krig") 
    call to_history("d2",dx,"flag_krig") 
  end if

  if (seq_sim .eq. 1) then

    call sep_write(rflag_seq_sim,"flag_sgs")
    call to_history("n1",nz,"flag_sgs") 
    call to_history("o1",oz,"flag_sgs") 
    call to_history("d1",dz,"flag_sgs") 
    call to_history("n2",nx,"flag_sgs")  
    call to_history("o2",ox,"flag_sgs") 
    call to_history("d2",dx,"flag_sgs") 
 
    call sep_write(ex_seq_cdf,"seq_cdf")
    call to_history("n1",ex_n_cdf,"seq_cdf")
    call to_history("o1",ex_o_cdf,"seq_cdf")
    call to_history("d1",ex_d_cdf,"seq_cdf")
    call to_history("n2",1,"seq_cdf")
    call to_history("o2",0.0,"seq_cdf")
    call to_history("d2",0.0,"seq_cdf")

    call sep_write(stand_dev_mat,"stand_dev_mat") 
    call to_history("n1",nz,"stand_dev_mat") 
    call to_history("o1",oz,"stand_dev_mat")
    call to_history("d1",dz,"stand_dev_mat")
    call to_history("n2",nx,"stand_dev_mat")
    call to_history("o2",ox,"stand_dev_mat")
    call to_history("d2",dx,"stand_dev_mat")
  end if

  call to_history("n1",nz,"flag_mc") 
  call to_history("o1",oz,"flag_mc") 
  call to_history("d1",dz,"flag_mc") 
  call to_history("n2",nx,"flag_mc")  
  call to_history("o2",ox,"flag_mc") 
  call to_history("d2",dx,"flag_mc") 

!  call to_history("n1",nz,"flag_sum") 
!  call to_history("o1",oz,"flag_sum") 
!  call to_history("d1",dz,"flag_sum") 
!  call to_history("n2",nx,"flag_sum")  
!  call to_history("o2",ox,"flag_sum") 
!  call to_history("d2",dx,"flag_sum")

!  call to_history("n1",nz,"flag_out") 
!  call to_history("o1",oz,"flag_out") 
!  call to_history("d1",dz,"flag_out") 
!  call to_history("n2",nx,"flag_out")  
!  call to_history("o2",ox,"flag_out") 
!  call to_history("d2",dx,"flag_out") 

!  call sep_write(rflag,"flag_out")

  call sep_write(rflag_mc,"flag_mc")
!  call sep_write(rflag_sum,"flag_sum")

  call sep_close()  

  call cpu_time(t_end) 
  write(0,*) "comp time",t_end - t_start 
  write(0,*) "mean sparse data",mean_sparse
  
end program
