module linear_algebra_mod

  implicit none

  contains

  function get_inverse_symmetric(matrix,n,errorflag) result(inverse)
    integer,intent(in)                            :: n
    integer,intent(out)                           :: errorflag ! returns -1 for error, 0 when the inversion works
    real,intent(in),dimension(:,:)                :: matrix
    real,dimension(:,:),allocatable               :: inverse
    logical                                       :: flag=.true.
    real                                          :: m
    integer                                       :: i,j,k,l
    real,dimension(:,:),allocatable               :: augmented_matrix

    allocate(augmented_matrix(n,2*n))
    allocate(inverse(n,n))

    ! augment input matrix with an identity matrix
    do i=1,n
      do j=1,2*n
        if (j <= n) then
          augmented_matrix(i,j) = matrix(i,j)
        else if ((i+n) == j) then
          augmented_matrix(i,j) = 1.
        else
          augmented_matrix(i,j)=0.
        end if
      end do
    end do
          
    ! Reduce augmented matrix to upper triangular form
    do k =1,n-1
      if (augmented_matrix(k,k) == 0) then
        flag = .false.
	do i = k+1, n
          if (augmented_matrix(i,k) /= 0) then
	    do j = 1,2*n
	      augmented_matrix(k,j) = augmented_matrix(k,j)+augmented_matrix(i,j)
	    end do
	    flag = .true.
	    exit
	  end if
	  if (flag .eqv. .false.) then
	    print*, "Matrix is non - invertible"
	    inverse = 0
	    errorflag = -1
	    return
	  end if
        end do
      end if	
      do j = k+1, n			
        m = augmented_matrix(j,k)/augmented_matrix(k,k)
	do i = k, 2*n
	  augmented_matrix(j,i) = augmented_matrix(j,i) - m*augmented_matrix(k,i)
	end do
      end do
    end do

    !Test for invertibility
    do i = 1,n
      if (augmented_matrix(i,i) == 0.) then
        print*, "Matrix is non - invertible"
	inverse = 0.
	errorflag = -1
	return
      end if
    end do
	
    !Make diagonal elements as 1
    do i=1,n
      m = augmented_matrix(i,i)
      do j = i,(2*n)				
        augmented_matrix(i,j) = (augmented_matrix(i,j) / m)
      end do 
    end do
	
    !Reduced right side half of augmented matrix to identity matrix
    do k=n-1,1,-1
      do i =1,k
        m = augmented_matrix(i,k+1)
        do j=k,(2*n)
	  augmented_matrix(i,j) = augmented_matrix(i,j) -augmented_matrix(k+1,j) * m
	end do
      end do
    end do				
	
    !store answer
    do i=1,n
      do j=1,n
        inverse(i,j) = augmented_matrix(i,j+n)
      end do
    end do	
    errorflag = 0
 
 end function

 function matrix_multiply(m1,m2) result(m3)
    ! multiplying m1xm2
    real,dimension(:,:)                :: m1,m2
    real,dimension(:,:),allocatable    :: m3
    integer                            :: i,j,k,e,f,g

    g = size(m1,1)
    f = size(m1,2)
    e = size(m2,2)

    allocate(m3(g,e))
    m3 = 0.
    do i=1,g
      do j=1,e
        do k=1,f
          m3(i,j) = m3(i,j) + m1(i,k) * m2(k,j)
        end do
      end do
    end do
   
  end function

end module
