program make_sparse_do_krigging

  use sep
  use make_sparse_mod
  use scan_mod

  implicit none

  integer                              :: nz,nx,nb_to_eliminate,i,j,k,ix,nb_eliminated,points_found,nb_known_samp,max_krig_samp,min_krig_samp
  real                                 :: oz,dz,ox,dx,perc,sill,range,nugget,alpha,rand_nb
  real,dimension(:,:),allocatable      :: data,samp_position
  integer,dimension(:,:),allocatable   :: flag,flag_krig,flag_mc,flag_start
  real                                 :: x_coord,z_coord
  integer                              :: n1,n2,n3,n4,n
  logical                              :: stop_krig
  
  call sep_init()

  call from_history("n1", nz) 
  call from_history("o1", oz)
  call from_history("d1", dz)
  call from_history("n2", nx) 
  call from_history("o2", ox)
  call from_history("d2", dx)

  allocate(sparse_data(nz,nx))
  call sep_read(sparse_data)

  allocate(flag(nz,nx))
  allocate(flag_krig(nz,nx))
  allocate(flag_mc(nz,nx))
  allocate(flag_start(nz,nx))
  flag=1

  !!!!!!!! variogram parameters  !!!!!!!!!
  call from_param("nugget",nugget)
  call from_param("sill",sill)
  call from_param("range",range) ! range has to be in number of samples (we neglect the fact that the sample diagonally next to the point is further way than the one next to it vertically or horizontally).
  call from_param("alpha",alpha)
  call from_param("max_krig_samp",max_krig_samp)
  call from_param("min_krig_samp",min_krig_samp)
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


 
  total_nb_points = nx * nz


  ! updating the number of known samples
  nb_known_samp = 0
  do i=1,nz
    do j=1,nx
      nb_known_samp = nb_known_samp + flag(i,j)
    end do
  end do
  
  ! updating the flag matrix for initial samples. flag_start is the points that we start with (I want to keep track of which point has been known from the start, kriged, or guessed by Monte Carlo)
  flag_start = flag 


  do while (nb_known_samp .lt. total_nb_points)

    ! 1. picking a new unknown point
    ! picking a random x coordinate
    call random_number(rand_nb)
    x_coord = ox + (rand_nb)*(nx-1)*dx
    j = 1.5 + (x_coord-ox)/dx

    ! picking a random z coordinate
    call random_number(rand_nb)
    z_coord = oz + (rand_nb)*(nz-1)*dz
    i = 1.5 + (z_coord-oz)/dz

    if (flag(i,j) == 0) then

        new_position(1) = i ; new_position(2) = j       
        stop_krig=0 ! set the flag to zero by default  

        call scan2d_1(range,current_position,n_min,n_max,flag,stop_krig,samp_q1) ! scan top left quadrant --> returns an array A(i,j) where i is the ith sample that will be used in the krigging process
        if (allocated(samp_q1)) then
          samp_position = samp_q1
        end if

        samp_q2 = scan2d(2,range,current_position,stop_krig) ! scan top right quadrant 
        if (allocated(samp_q2)) then
          samp_position = [samp_position,samp_q2]  
        end if

        samp_q3 = scan2d(3,range,current_position,stop_krig) ! scan bottom left quadrant
        if (allocated(samp_q3)) then
          samp_position = [samp_position,samp_q3]
        end if

        samp_q4 = scan2d(4,range,current_position,stop_krig) ! scan bottom right quadrant
        if (allocated(samp_q4)) then 
          samp_position = [samp_position,samp_q4]	
        end if

        n = size(samp_position,1)
        ! stop_krig is an integer: each time we scan a quadrant, we increment it by one only if we are satisfied with the scanned region (ie if there are enough points that enable us to procede to kriging).

        if ((do_krig .eqv. 4) .and. (n .ge. min_krig_samp)) then 
          allocate(samp_values(n))
          do i=1,n
            samp_values(i) = data(samp_position(i,1),samp_position(i,2))
          end do          
          data(i,j) = get_kriged_value(samp_position,samp_values,new_position,covariance)
          flag_krig(i,j) = 1 ! indicating that this point has been created by krigging
        else ! otherwise, pick value with Monte Carlo
          data(i,j) = generate_random_val(cdf)
          flag_mc(i,j) = 1 ! indicating that this point has been created by MC
        end if
        flag(i,j) = 1
        nb_known_samp = nb_known_samp + 1

    end if

  call to_history("n1", nz) 
  call to_history("o1", oz)
  call to_history("d1", dz)
  call to_history("n2", nx) 
  call to_history("o2", ox)
  call to_history("d2", dx)
  call sep_write(data)

  call sep_close()    
  
end program
