! Kirchhoff modeling and migration.
program Kapp

  use sep
  use Kirchhoff

  implicit none

  real, allocatable, dimension(:,:) :: modl, data
  real, allocatable, dimension(:)   :: vrms
  integer                           :: n1, n2, n3
  real                              :: o1, d1, o2, d2, o3, d3
  integer                           :: i, istart, iend, iangle
  real                              :: velocity, angle, aperture
  logical                           :: adj, isangle

  call sep_init()

  call from_history("n1", n1)
  call from_history("o1", o1)
  call from_history("d1", d1)
  call from_history("n2", n2)
  call from_history("o2", o2)
  call from_history("d2", d2)

  call from_param("adj", adj)
  call from_param("isangle", isangle, .true.)
  call from_param("velocity", velocity, 1.5)
  call from_param("istart", istart, 5)
  call from_param("iend", iend, 90)
  call from_param("iangle", iangle, 5)

  n3 = (iend-istart)/iangle + 1

  call to_history("n1", n1)
  call to_history("o1", o1)
  call to_history("d1", d1)
  call to_history("n2", n2)
  call to_history("o2", o2)
  call to_history("d2", d2)
  call to_history("n3", n3)
  call to_history("o3", istart)
  call to_history("d3", iangle)

  allocate(modl(n1,n2))
  allocate(data(n1,n2))
  allocate(vrms(n1))
  modl = 0.
  data = 0.
  vrms = 0.

  call sep_read(vrms, "vel")
  vrms=vrms/2.
  if(adj) then
    call sep_read(data)
  else
    call sep_read(modl)
  endif

  call kirchfast_init(vrms, o1, d1, d2, n1, n2, isangle)

  if(isangle) then
    do i=istart,iend,iangle
      angle=i
      write(0,*) "limit angles with ", i/iangle, angle
      call update_limit(angle, aperture)
      call kirchfast(adj, .false., modl, data)
      if(adj) then
        call sep_write(modl)
      else
        call sep_write(data)
      endif
    enddo
  else 
    do i=istart,iend,iangle
      aperture=i
      write(0,*) "limit apertures with ", i/iangle, aperture
      call update_limit(angle, aperture)
      call kirchfast(adj, .false., modl, data)
      if(adj) then
        call sep_write(modl)
      else
        call sep_write(data)
      endif
    enddo
  endif

  call sep_close()

end program
