program old_kriging_program

  use sep
  use scan_mod
  use kriging_mod
  use stats_mod

  implicit none

  integer                              :: nz,nx,n,range,n_min,n_max,nb1,nb2,nb3,nb4,known_samp,nb_samp_krig
  real                                 :: oz,dz,ox,dx,z_coord,x_coord
  real,dimension(:,:),allocatable      :: sparse_data,temp
  integer,dimension(:,:),allocatable   :: flag,flag_krig,flag_mc,flag_start,flag_seq_sim
  real,dimension(:,:),allocatable      :: rflag,rflag_krig,rflag_mc,rflag_start,rflag_sum,rflag_seq_sim
  real                                 :: rand_nb
  logical                              :: stop_krig
  integer,dimension(2)                 :: position
  integer                              :: i,j,k,by_krig,by_mc,beginning,at_end
  real                                 :: nugget,range_var,sill,alpha
  real,dimension(:),allocatable        :: samp_values,cdf,seq_cdf
  integer,dimension(:,:),allocatable   :: samp_position,samp1,samp2,samp3,samp4
  integer                              :: counter,niter,n_cdf,n_stop,seq_sim,seq_n_cdf,nb_stand_dev
  real                                 :: sigma,o_cdf,d_cdf,avg,power,error_var
  real                                 :: temp_krig_value,error_stand_dev,seq_o_cdf,seq_f_cdf,seq_d_cdf


  call sep_init()

  call from_history("n1", nz) 
  call from_history("o1", oz)
  call from_history("d1", dz)
  call from_history("n2", nx) 
  call from_history("o2", ox)
  call from_history("d2", dx)

  !!!!!! parameters for scanning !!!!!!
  call from_param("n_max",n_max) 
  call from_param("n_min",n_min)
  call from_param("range",range)

  !!!!!! parameters for variogram !!!!!!
  call from_param("nugget",nugget)
  call from_param("range_variogram",range_var)
  call from_param("sill",sill)  !sill is the variance for our stationary model
  call from_param("alpha",alpha)
  call from_param("niter",niter)
  call from_param("avg",avg) ! import the mean for our stationary model
  call from_param("power",power)

   !!!!!! parameters for CDF !!!!!!
  call from_param("n_cdf",n_cdf)
  call from_param("o_cdf",o_cdf)
  call from_param("d_cdf",d_cdf)

  !!!!!!! parameter for Gaussian sequential simulation !!!!!!! 
  call from_param("seq_sim",seq_sim)
  call from_param("nb_stand_dev",nb_stand_dev)

  call ordinary_kriging_init(nugget,range_var,sill,alpha,power,niter) !!!! initializing the ordinary kriging function

  !!!!!!!!!!!!!!!!!!!!!!!!!!! generating the cumulative density function (CDF) !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  sigma = sqrt(sill)
  allocate(cdf(n_cdf))
  cdf = gaussian_cdf(o_cdf,d_cdf,n_cdf,avg,sigma)

  allocate(sparse_data(nz,nx))
  allocate(flag(nz,nx))
  allocate(temp(nz,nx))
  allocate(samp1(n_max,2))
  allocate(samp2(n_max,2))
  allocate(samp3(n_max,2))
  allocate(samp4(n_max,2))

  call sep_read(sparse_data) 
  call sep_read(temp,"flag") ! flag_matrix.H is an auxiliary file
  flag = temp ! flag has to be of type integer, whereas temp is the array read using sep_read (so we need to convert it from real -> integer).
  deallocate(temp)

  allocate(flag_krig(nz,nx)) !flag_krig=1 when the value has been determined using kriging
  allocate(flag_mc(nz,nx)) ! flag_mc = 1 when the value has been generated usign MC
  allocate(flag_start(nz,nx)) ! flag_start=1 when the value was known from the start
  allocate(flag_seq_sim(nz,nx)) !flag for Gaussian sequential simulation
  allocate(rflag(nz,nx)) ! flag=1 when the point has a known value
  allocate(rflag_krig(nz,nx)) !flag_krig=1 when the value has been determined using kriging
  allocate(rflag_mc(nz,nx)) ! flag_mc = 1 when the value has been generated usign MC
  allocate(rflag_start(nz,nx)) ! flag_start=1 when the value was known from the start
  allocate(rflag_sum(nz,nx))
  allocate(rflag_seq_sim(nz,nx)) !flag for Gaussian sequential simulation

  !!!!!!!!  saving the starting flag array in flag_start !!!!!!!! 
  flag_start = flag 
  flag_krig = 0
  flag_mc = 0
  flag_seq_sim = 0
 
  !!!!!!!! getting the original nb of known samples !!!!!!!!
  known_samp = 0
  do i=1,nz
    do j=1,nx
      known_samp = known_samp + flag(i,j)
    end do
  end do

  !!!!!!!!  starting sample generation process !!!!!!!!
  n = nz * nx
  counter = 0

  do while ((known_samp .lt. n)) ! full interpolation
  !do while ((known_samp .lt. n) .and. (counter .lt. 4)) ! interpolate only for a limited number of points
    counter = counter + 1
    write(0,*) "nb",known_samp
    write(0,*) "remaining to interpolate", n - known_samp

    call random_number(rand_nb)
    x_coord = ox + (rand_nb)*(nx-1)*dx
    j = 1.5 + (x_coord-ox)/dx

    call random_number(rand_nb)
    z_coord = oz + (rand_nb)*(nz-1)*dz
    i = 1.5 + (z_coord-oz)/dz

    if (flag(i,j) .eq. 0) then

        samp1=0;samp2=0;samp3=0;samp4=0
        nb1=0;nb2=0;nb3=0;nb4=0

        position(1) = i ; position(2) = j       
        stop_krig= .false.
        !write(0,*) "pos",position(1),position(2)

        call scan2d_1(range,position,n_min,n_max,flag,stop_krig,samp1,nb1) !scan top left
        if (stop_krig .eqv. .false.) then
          call scan2d_2(range,position,n_min,n_max,flag,stop_krig,samp2,nb2) !scan top right
          if (stop_krig .eqv. .false.) then
            call scan2d_3(range,position,n_min,n_max,flag,stop_krig,samp3,nb3) !scan bottom right
            if (stop_krig .eqv. .false.) then
              call scan2d_4(range,position,n_min,n_max,flag,stop_krig,samp4,nb4) !scan bottom left
            end if
          end if
        end if
        nb_samp_krig = nb1 + nb2 + nb3 + nb4

        if ((stop_krig .eqv. .false.) .and. (nb_samp_krig .ge. n_min)) then

          allocate(samp_position(nb_samp_krig,2))
          allocate(samp_values(nb_samp_krig))

          !write(0,*) "nb1",nb1
          !write(0,*) "nb2",nb2
          !write(0,*) "nb3",nb3
          !write(0,*) "nb4",nb4
          !write(0,*) "total nb", nb_samp_krig

          !!!!!!!! copying the sample positions from each quadrant to one array called samp_position   
          !!!!!!!! poorly coded - need to ask Bob how to concatenate the four matrices all together  

          if (nb1 .gt. 0) then
            do k=1,nb1
              !write(0,*) "samp1(",k,",1) = ",samp1(k,1)
              !write(0,*) "samp1(",k,",2) = ",samp1(k,2)
              !write(0,*) "val1(",k,",2) = ",sparse_data(samp1(k,1),samp1(k,2))
            end do
            do k=1,nb1
            samp_position(k,1) = samp1(k,1)
            samp_position(k,2) = samp1(k,2)
            samp_values(k)     = sparse_data(samp1(k,1),samp1(k,2))
            end do
          end if
          if (nb2 .gt. 0) then
            do k=1,nb2
              !write(0,*) "samp2(",k,",1) = ",samp2(k,1)
              !write(0,*) "samp2(",k,",2) = ",samp2(k,2)
              !write(0,*) "val2(",k,",2) = ",sparse_data(samp2(k,1),samp2(k,2))
            end do
            do k=1,nb2
              samp_position(k+nb1,1) = samp2(k,1)
              samp_position(k+nb1,2) = samp2(k,2)
              samp_values(k+nb1)     = sparse_data(samp2(k,1),samp2(k,2))
 
            end do
          end if
          if (nb3 .gt. 0) then
            do k=1,nb3
              !write(0,*) "samp3(",k,",1) = ",samp3(k,1)
              !write(0,*) "samp3(",k,",2) = ",samp3(k,2)
              !write(0,*) "val3(",k,",2) = ",sparse_data(samp3(k,1),samp3(k,2))
            end do
            do k=1,nb3
              samp_position(k+nb1+nb2,1) = samp3(k,1)
              samp_position(k+nb1+nb2,2) = samp3(k,2)
              samp_values(k+nb1+nb2)     = sparse_data(samp3(k,1),samp3(k,2))
            end do
          end if
          if (nb4 .gt. 0) then
          do k=1,nb4
            !write(0,*) "samp4(",k,",1) = ",samp4(k,1)
            !write(0,*) "samp4(",k,",2) = ",samp4(k,2)
            !write(0,*) "val4(",k,",2) = ",sparse_data(samp4(k,1),samp4(k,2))
          end do
            do k=1,nb4
              samp_position(k+nb1+nb2+nb3,1) = samp4(k,1)
              samp_position(k+nb1+nb2+nb3,2) = samp4(k,2)
              samp_values(k+nb1+nb2+nb3)     = sparse_data(samp4(k,1),samp4(k,2))
            end do
          end if

          if (seq_sim .eq. 0) then
            !!!!!!!! kriging with no sequential simulation 
            sparse_data(i,j) = ordinary_kriging(samp_position,samp_values,position,error_var)
            !write(0,*) "kriged value",sparse_data(i,j)
            flag_krig(i,j) = 1
            !write(0,*) "finshed kriging, counter",counter
            deallocate(samp_position)
            deallocate(samp_values)

          else ! kriging with Gaussian simulation 

            temp_krig_value = ordinary_kriging(samp_position,samp_values,position,error_stand_dev)
            
            seq_o_cdf = temp_krig_value - nb_stand_dev * error_stand_dev
            seq_f_cdf = temp_krig_value - nb_stand_dev * error_stand_dev
            seq_d_cdf = (seq_f_cdf - seq_o_cdf) / seq_n_cdf        

            allocate(seq_cdf(n_cdf))    
            seq_cdf = gaussian_cdf(seq_o_cdf,seq_d_cdf,seq_n_cdf,temp_krig_value,error_stand_dev)

            sparse_data(i,j) = mc_gaussian(seq_cdf,seq_o_cdf,seq_d_cdf,seq_n_cdf)
            flag_seq_sim(i,j) = 1

            deallocate(samp_position)
            deallocate(samp_values)
            deallocate(seq_cdf)       
  
          end if

        else
          sparse_data(i,j) = mc_gaussian(cdf,o_cdf,d_cdf,n_cdf)
          flag_mc(i,j) = 1
          !write(0,*) "value generated by MC"
        end if

        flag(i,j) = 1
        known_samp = known_samp + 1

    end if
  end do

  rflag_sum = 1.0 * flag_start + 1.0 * flag_krig 
  rflag = 1.0 * flag
  rflag_krig = 1.0 * flag_krig
  rflag_mc = 1.0 * flag_mc

  !!!!!! displaying the nb of points for each method

  by_krig=0;by_mc=0;at_end=0;beginning=0
  do i=1,nz
    do j=1,nx
      by_krig   = by_krig   + flag_krig(i,j)
      by_mc     = by_mc     + flag_mc(i,j)
      beginning = beginning + flag_start(i,j)
      at_end    = at_end    + flag(i,j)
    end do
  end do
  write(0,*) "total points", n
  write(0,*) "krig:",by_krig
  write(0,*) "mc:",by_mc
  write(0,*) "beginning:",beginning
  write(0,*) "end:",at_end

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! output !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  
  call sep_write(sparse_data)
  call sep_write(rflag,"flag_out")
  call sep_write(rflag_krig,"flag_krig")
  call sep_write(rflag_mc,"flag_mc")
  call sep_write(rflag_sum,"flag_sum")

  call to_history("n1",nz) 
  call to_history("o1",oz)
  call to_history("d1",dz)
  call to_history("n2",nx) 
  call to_history("o2",ox)
  call to_history("d2",dx)

  call to_history("n1",nz,"flag_out") 
  call to_history("o1",oz,"flag_out") 
  call to_history("d1",dz,"flag_out") 
  call to_history("n2",nx,"flag_out")  
  call to_history("o2",ox,"flag_out") 
  call to_history("d2",dx,"flag_out") 

  call to_history("n1",nz,"flag_krig") 
  call to_history("o1",oz,"flag_krig") 
  call to_history("d1",dz,"flag_krig") 
  call to_history("n2",nx,"flag_krig")  
  call to_history("o2",ox,"flag_krig") 
  call to_history("d2",dx,"flag_krig") 

  call to_history("n1",nz,"flag_mc") 
  call to_history("o1",oz,"flag_mc") 
  call to_history("d1",dz,"flag_mc") 
  call to_history("n2",nx,"flag_mc")  
  call to_history("o2",ox,"flag_mc") 
  call to_history("d2",dx,"flag_mc") 

  call to_history("n1",nz,"flag_sum") 
  call to_history("o1",oz,"flag_sum") 
  call to_history("d1",dz,"flag_sum") 
  call to_history("n2",nx,"flag_sum")  
  call to_history("o2",ox,"flag_sum") 
  call to_history("d2",dx,"flag_sum")

  call sep_close()    
  
end program
