program detrend

  use sep
  implicit none

! 1. for all tau's, pick the max value on semblance plot, which gives us a rough curve as a function of traveltime depth
! 2. keep only the low frequencies of this curve => we obtain a trend
! 3. correct the semblance plot by taking out the trend

  integer                               :: ntau,nv,i,j,fairway_width,mid
  real                                  :: otau,dtau,ov,dv,time
  real,dimension(:),allocatable         :: max_vel,cap_vel,floor_vel,rmax_index,rfloor_index,rcap_index
  integer,dimension(:),allocatable      :: max_index,cap_index,floor_index
  real,dimension(:,:),allocatable       :: semblance
  real                                  :: a3,a2,a1,a0,b3,b2,b1,b0

  call sep_init()

  call from_history("n1",ntau)
  call from_history("o1",otau)
  call from_history("d1",dtau)

  call from_history("n2",nv)
  call from_history("o2",ov)
  call from_history("d2",dv)

  call from_param("a0",a0)
  call from_param("a1",a1)
  call from_param("a2",a2)
  call from_param("a3",a3)

  call from_param("b0",b0)
  call from_param("b1",b1)
  call from_param("b2",b2)
  call from_param("b3",b3)

  allocate(semblance(ntau,nv)) 
  allocate(max_index(ntau))
  allocate(rmax_index(ntau))
  allocate(max_vel(ntau))
  allocate(cap_index(ntau))
  allocate(cap_vel(ntau))
  allocate(floor_index(ntau))
  allocate(floor_vel(ntau))
  allocate(rfloor_index(ntau))
  allocate(rcap_index(ntau))

  call sep_read(semblance)

  do i=1,ntau
    time = otau + (i-1) * dtau 
    cap_vel(i) = a3 * time**3 + a2 * time**2 + a1 * time + a0 
    floor_vel(i) = b3 * time**3 + b2 * time**2 + b1 * time + b0 
  end do
  
  do i=1,ntau
    cap_index(i) = 1
    floor_index(i) = 1
    do while ( (ov + (cap_index(i)-1)*dv) .lt. cap_vel(i)) 
      cap_index(i) = cap_index(i) + 1 
    end do
    do while ( (ov + (floor_index(i)-1)*dv) .lt. floor_vel(i)) 
      floor_index(i) = floor_index(i) + 1 
    end do
  end do

  do i=1,ntau
    do j=cap_index(i),nv
      semblance(i,j) = 0
    end do
    do j=1,floor_index(i)
      semblance(i,j) = 0
    end do
  end do

  do i=1,ntau
    max_index(i) = maxloc(semblance(i,:),dim=1)
    max_vel(i)   = ov + (max_index(i)-1) * dv
    write(0,*) "max index",i,max_index(i)
    write(0,*) "max vel",i,max_vel(i)
  end do

 ! do i=1,10
 !   max_vel(i) = 1.5
 ! end do

  mid = ntau/2
  fairway_width = cap_index(mid) - floor_index(mid) 
  write(0,*) "fairway width in nb of samples",fairway_width

  rmax_index = max_index
  rfloor_index = floor_index
  rcap_index = cap_index

  call sep_write(max_vel)
  call to_history("n1",ntau)
  call to_history("o1",otau)
  call to_history("d1",dtau)
  call to_history("n2",1)
  call to_history("o2",0.0)
  call to_history("d2",0.0)

  call sep_write(rmax_index,"peak_index")
  call to_history("n1",ntau,"peak_index")
  call to_history("o1",otau,"peak_index")
  call to_history("d1",dtau,"peak_index")
  call to_history("n2",1,"peak_index")
  call to_history("o2",0.0,"peak_index")
  call to_history("d2",0.0,"peak_index")

  call sep_write(rfloor_index,"floor_index")
  call to_history("n1",ntau,"floor_index")
  call to_history("o1",otau,"floor_index")
  call to_history("d1",dtau,"floor_index")
  call to_history("n2",1,"floor_index")
  call to_history("o2",0.0,"floor_index")
  call to_history("d2",0.0,"floor_index")

  call sep_write(rcap_index,"cap_index")
  call to_history("n1",ntau,"cap_index")
  call to_history("o1",otau,"cap_index")
  call to_history("d1",dtau,"cap_index")
  call to_history("n2",1,"cap_index")
  call to_history("o2",0.0,"cap_index")
  call to_history("d2",0.0,"cap_index")

  call sep_write(semblance,"muted_semblance")
  call to_history("n1",ntau,"muted_semblance")
  call to_history("o1",otau,"muted_semblance")
  call to_history("d1",dtau,"muted_semblance")
  call to_history("n2",nv,"muted_semblance")
  call to_history("o2",ov,"muted_semblance")
  call to_history("d2",dv,"muted_semblance")

  call sep_close()    

end program
