program detrend_2d

  use sep
  implicit none

  integer                               :: ntau,nv,ncmp,i,j,fairway_width,mid,k
  real                                  :: otau,dtau,ov,dv,time,ocmp,dcmp
  real,dimension(:,:),allocatable       :: max_vel,cap_vel,floor_vel,rmax_index,rfloor_index,rcap_index
  integer,dimension(:,:),allocatable    :: max_index,cap_index,floor_index
  real,dimension(:,:,:),allocatable     :: semblance
  real                                  :: a3,a2,a1,a0,b3,b2,b1,b0

  call sep_init()

  call from_history("n1",ntau)
  call from_history("o1",otau)
  call from_history("d1",dtau)

  call from_history("n2",nv)
  call from_history("o2",ov)
  call from_history("d2",dv)

  call from_history("n3",ncmp)
  call from_history("o3",ocmp)
  call from_history("d3",dcmp)

  call from_param("a0",a0)
  call from_param("a1",a1)
  call from_param("a2",a2)
  call from_param("a3",a3)

  call from_param("b0",b0)
  call from_param("b1",b1)
  call from_param("b2",b2)
  call from_param("b3",b3)

  allocate(semblance(ntau,nv,ncmp)) 
  allocate(max_index(ntau,ncmp))
  allocate(rmax_index(ntau,ncmp))
  allocate(max_vel(ntau,ncmp))
  allocate(cap_index(ntau,ncmp))
  allocate(cap_vel(ntau,ncmp))
  allocate(floor_index(ntau,ncmp))
  allocate(floor_vel(ntau,ncmp))
  allocate(rfloor_index(ntau,ncmp))
  allocate(rcap_index(ntau,ncmp))

  call sep_read(semblance)

  do k=1,ncmp
    do i=1,ntau
      time = otau + (i-1) * dtau 
      cap_vel(i,k)   = a3 * time**3 + a2 * time**2 + a1 * time + a0 
      floor_vel(i,k) = b3 * time**3 + b2 * time**2 + b1 * time + b0 
    end do
  end do
 
  do k=1,ncmp
    do i=1,ntau
      cap_index(i,k) = 1
      floor_index(i,k) = 1
      do while ( (ov + (cap_index(i,k)-1)*dv) .lt. cap_vel(i,k)) 
        cap_index(i,k) = cap_index(i,k) + 1 
      end do
      do while ( (ov + (floor_index(i,k)-1)*dv) .lt. floor_vel(i,k)) 
        floor_index(i,k) = floor_index(i,k) + 1 
      end do
    end do
  end do

  do k=1,ncmp
    do i=1,ntau
      do j=cap_index(i,k),nv
        semblance(i,j,k) = 0
      end do
      do j=1,floor_index(i,k)
        semblance(i,j,k) = 0
      end do
    end do
  end do
 
  do k=1,ncmp
    do i=1,ntau
      max_index(i,k) = maxloc(semblance(i,:,k),dim=1)
      max_vel(i,k)   = ov + (max_index(i,k)-1) * dv
    end do
  end do

  rmax_index = max_index
  rfloor_index = floor_index
  rcap_index = cap_index

  call sep_write(max_vel)
  call to_history("n1",ntau)
  call to_history("o1",otau)
  call to_history("d1",dtau)
  call to_history("n2",ncmp)
  call to_history("o2",ocmp)
  call to_history("d2",dcmp)
  call to_history("n3",1)
  call to_history("o3",0.0)
  call to_history("d3",0.0)

  call sep_write(rmax_index,"peak_index")
  call to_history("n1",ntau,"peak_index")
  call to_history("o1",otau,"peak_index")
  call to_history("d1",dtau,"peak_index")
  call to_history("n2",ncmp,"peak_index")
  call to_history("o2",ocmp,"peak_index")
  call to_history("d2",dcmp,"peak_index")
  call to_history("n3",1,"peak_index")
  call to_history("o3",0.0,"peak_index")
  call to_history("d3",0.0,"peak_index")

  call sep_write(rfloor_index,"floor_index")
  call to_history("n1",ntau,"floor_index")
  call to_history("o1",otau,"floor_index")
  call to_history("d1",dtau,"floor_index")
  call to_history("n2",ncmp,"floor_index")
  call to_history("o2",ocmp,"floor_index")
  call to_history("d2",dcmp,"floor_index")
  call to_history("n3",1,"peak_index")
  call to_history("o3",0.0,"peak_index")
  call to_history("d3",0.0,"peak_index")

  call sep_write(rcap_index,"cap_index")
  call to_history("n1",ntau,"cap_index")
  call to_history("o1",otau,"cap_index")
  call to_history("d1",dtau,"cap_index")
  call to_history("n2",ncmp,"cap_index")
  call to_history("o2",ocmp,"cap_index")
  call to_history("d2",dcmp,"cap_index")
  call to_history("n3",1,"peak_index")
  call to_history("o3",0.0,"peak_index")
  call to_history("d3",0.0,"peak_index")

  call sep_write(semblance,"muted_semblance")
  call to_history("n1",ntau,"muted_semblance")
  call to_history("o1",otau,"muted_semblance")
  call to_history("d1",dtau,"muted_semblance")
  call to_history("n2",nv,"muted_semblance")
  call to_history("o2",ov,"muted_semblance")
  call to_history("d2",dv,"muted_semblance")
  call to_history("n3",ncmp,"muted_semblance")
  call to_history("o3",1,"muted_semblance")
  call to_history("d3",1,"muted_semblance")

  call sep_close()    

end program
