! this program generates a 2D model. The value at each point of the model is pseudo-randomly generated by a MC method, using a Gaussian cdf.
program random_gaussian_model
  
  use sep
  use stats_mod

  implicit none
  
  real                               :: rand,o_cdf,d_cdf,o_samp,d_samp,p_min,p_max,mean,avg,sig,test
  integer                            :: n1,n2,i,j,n_cdf,nb_picks
  real,dimension(:,:),allocatable    :: model
  real,dimension(:),allocatable      :: cdf,coord
  real                               :: sample_avg,sample_stand_dev,o1,o2,d1,d2,sill

  call sep_init()

  call from_param("n1",n1)
  call from_param("n2",n2)
  call from_param("sill",sill) ! importing the variance for the Gaussian cdf model
  call from_param("avg",avg) ! importing the average for the Gaussian cdf model

  call from_param("n_cdf",n_cdf) ! importing the length of the cdf function
  call from_param("o_cdf",o_cdf) ! importing the starting point of the cdf function
  call from_param("d_cdf",d_cdf) ! importing the increment of the cdf function

  o1 = 1.0
  d1 = 1.0
  o2 = 1.0
  d2 = 1.0
  
  sig = sqrt(sill)

  allocate(model(n1,n2))
  allocate(cdf(n_cdf))

  write(0,*) "sig",sig
  write(0,*) "avg",avg
  
  ! building the cdf
  cdf = gaussian_cdf(o_cdf,d_cdf,n_cdf,avg,sig)
  !do i=1,size(cdf)
  !  write(0,*) "cdf", cdf(i)
  !end do

  p_min = cdf(1)
  p_max = cdf(n_cdf)

  write(0,*),"p min",p_min
  write(0,*),"p max",p_max

  ! generating the model
  do i=1,n1
    do j=1,n2
      call random_number(test)
      model(i,j) = mc_gaussian(cdf,o_cdf,d_cdf,n_cdf)
    end do
  end do

  ! calculating mean and variance of the generated model (just a quick check).
  sample_avg = average(model)
  sample_stand_dev = stand_dev(model,sample_avg)
  write(0,*) "mean of generated model",sample_avg
  write(0,*) "standard dev of generated model",sample_stand_dev

  call to_history("n1",n_cdf,"cdf")
  call to_history("o1",o_cdf,"cdf")
  call to_history("d1",d_cdf,"cdf")

  call to_history("n2",1,"cdf")
  call to_history("o2",0.0,"cdf")
  call to_history("d2",0.0,"cdf")

  !call to_history("n1",n_cdf,"coord")
  !call to_history("o1",o_cdf,"coord")
  !call to_history("d1",d_cdf,"coord")

  !call to_history("n2",1,"coord")
  !call to_history("o2",0.0,"coord")
  !call to_history("d2",0.0,"coord")

  call to_history("n1",n1)
  call to_history("o1",o1)
  call to_history("d1",d1)

  call to_history("n2",n2)
  call to_history("o2",o2)
  call to_history("d2",d2)
  
  call sep_write(model)
  !call sep_write(coord,"coord")
  call sep_write(cdf,"cdf")

  call sep_close()  

end program

