program raw_covariance
 
  use sep
  use covariance_mod
  use stats_mod

  implicit none
  
  integer                           :: nz,nx,dir_z,dir_x,n_lag,size_cov,i,n,norm
  real,dimension(:,:),allocatable   :: data 
  integer,dimension(:),allocatable  :: nb_pairs_vector
  real,dimension(:,:),allocatable   :: covariance
  real                              :: oz,dz,ox,dx,new_o1,new_d1,final_x   
  integer,dimension(2)              :: direction
  real,dimension(:),allocatable     :: lag_conversion
  real                              :: sigma,avg

  call sep_init()

  call from_history("n1", nz) 
  call from_history("o1", oz)
  call from_history("d1", dz)
  call from_history("n2", nx)
  call from_history("o2", ox)
  call from_history("d2", dx)

  n = max(nz,nx)

  call from_param("norm",norm) ! =1 if we want to normalize the output
  call from_param("dir_z",dir_z)
  call from_param("dir_x",dir_x)
  call from_param("length",size_cov,n) ! by default, the size of the covariance function is the max(n1,n2)
  direction(1) = dir_z
  direction(2) = dir_x

  write(0,*) "direction z", direction(1)
  write(0,*) "direction x", direction(2)

  allocate(data(nz,nx))
  call sep_read(data)
   
  allocate(covariance(size_cov,2))
  allocate(lag_conversion(size_cov))
  allocate(nb_pairs_vector(size_cov))

  covariance = 0.
  nb_pairs_vector = 0
  lag_conversion = 0.

  call covariance_function(data,direction,nb_pairs_vector,lag_conversion,covariance(:,1))
  covariance(:,2) = nb_pairs_vector 

  avg = average(data)
  sigma = stand_dev(data,avg)

  write(0,*) "data mean",avg
  write(0,*) "data standard dev",sigma  

  write(0,*) "covariance(1)",covariance(1,1)
!  write(0,*) "covariance(2)",covariance(2,1)
!  write(0,*) "covariance(3)",covariance(3,1)
!  write(0,*) "covariance(4)",covariance(4,1)
!  write(0,*) "covariance(5)",covariance(5,1)
 
  !do i=1,max(nz,nx)
  !  write(0,*) "lag",lag_conversion(i)
  !end do
  
  new_d1 = lag_conversion(2) - lag_conversion(1)
  new_o1 = lag_conversion(1)
  final_x = lag_conversion(size_cov) 

  if (norm .eq. 1) then
    covariance(:,1) = covariance(:,1)/covariance(1,1)
  end if

  write(0,*) "new d1",new_d1
  write(0,*) "new o1",new_o1
!  write(0,*) "final value",final_x 

  call to_history("n1",size_cov)
  call to_history("d1",new_d1)
  call to_history("o1",new_o1)

  call to_history("n2",2)
  call to_history("o2",0)
  call to_history("d2",1)

!  call to_history("n1",size_cov,"lag_conversion")
!  call to_history("d1",1.0,"lag_conversion")
!  call to_history("o1",0.0,"lag_conversion")

!  call to_history("n2",1,"lag_conversion")
!  call to_history("o2",0.0,"lag_conversion")
!  call to_history("d2",0.0,"lag_conversion")

  call to_history("label2","Covariance value")
  call to_history("label1","Number of samples")
  call to_history("title","Raw covariance function")

!  call to_history("label2","Covariance","lag_conversion")
!  call to_history("label1","nb of samples","lag_conversion")
!  call to_history("title","Covariance function","lag_conversion")

  call sep_write(covariance)
!  call sep_write(lag_conversion,"lag_conversion")

  call sep_close()  

end program


