module scan_mod
! this module contains 6 subroutines: 4 subroutines for 2d (each of them scan one quadrant), 2 for 1d
  implicit none
  
  contains

  subroutine scan2d(range,position,n_min,n_max,flag,stop_krig,samp_position,samp_values,sparse_data)

    integer                                  :: range,n_min,n_max,nb1,nb2,nb3,nb4,nb_samp_krig,k
    integer,dimension(2)                     :: position   
    integer,dimension(:,:)                   :: flag
    logical                                  :: stop_krig
    real,dimension(:,:)                      :: sparse_data
    real,dimension(:),allocatable            :: samp_values
    integer,dimension(:,:),allocatable       :: samp1,samp2,samp3,samp4,samp_position

    !!!! allocate sampi and nbi
    allocate(samp1(n_max,2))
    allocate(samp2(n_max,2))
    allocate(samp3(n_max,2))
    allocate(samp4(n_max,2))

    !!!! initialize sampi and nbi
    samp1=0;samp2=0;samp3=0;samp4=0
    nb1=0;nb2=0;nb3=0;nb4=0

    stop_krig = .false.
    
    call scan2d_1(range,position,n_min,n_max,flag,stop_krig,samp1,nb1) !scan top left
    if (stop_krig .eqv. .false.) then
      call scan2d_2(range,position,n_min,n_max,flag,stop_krig,samp2,nb2) !scan top right
      if (stop_krig .eqv. .false.) then
        call scan2d_3(range,position,n_min,n_max,flag,stop_krig,samp3,nb3) !scan bottom right
        if (stop_krig .eqv. .false.) then
          call scan2d_4(range,position,n_min,n_max,flag,stop_krig,samp4,nb4) !scan bottom left
        end if
      end if
    end if

    nb_samp_krig = nb1 + nb2 + nb3 + nb4

    if ((stop_krig .eqv. .false.) .and. (nb_samp_krig .ge. n_min)) then ! we do kriging
      allocate(samp_position(nb_samp_krig,2))
      allocate(samp_values(nb_samp_krig))


      if (nb1 .gt. 0) then ! copy samples from first quadrant
        do k=1,nb1
          samp_position(k,1) = samp1(k,1)
          samp_position(k,2) = samp1(k,2)
          samp_values(k)     = sparse_data(samp1(k,1),samp1(k,2))
          end do
      end if

      if (nb2 .gt. 0) then ! copy samples from second quadrant
        do k=1,nb2
          samp_position(k+nb1,1) = samp2(k,1)
          samp_position(k+nb1,2) = samp2(k,2)
          samp_values(k+nb1)     = sparse_data(samp2(k,1),samp2(k,2))
        end do
      end if

      if (nb3 .gt. 0) then
        do k=1,nb3
          samp_position(k+nb1+nb2,1) = samp3(k,1)
          samp_position(k+nb1+nb2,2) = samp3(k,2)
          samp_values(k+nb1+nb2)     = sparse_data(samp3(k,1),samp3(k,2))
        end do
      end if

      if (nb4 .gt. 0) then
        do k=1,nb4
          samp_position(k+nb1+nb2+nb3,1) = samp4(k,1)
          samp_position(k+nb1+nb2+nb3,2) = samp4(k,2)
          samp_values(k+nb1+nb2+nb3)     = sparse_data(samp4(k,1),samp4(k,2))
        end do
      end if

      !!!! deallocate sampi and nbi
      deallocate(samp1)
      deallocate(samp2)
      deallocate(samp3)
      deallocate(samp4)

    else ! case where stop_krig = true or if there is not enough points
      stop_krig = .true.

    end if
          
  end subroutine

  subroutine scan2d_1(range,position,n_min,n_max,flag,stop_krig,points,nb_samples) !scanning top left corner
    integer                             :: range,n_min,n_max,lag2,lag1,i,j,max_lag1,max_lag2,radius,max_lag
    integer,allocatable,dimension(:,:)  :: temp
    integer,dimension(2)                :: position !!!!give the position in sample number!!!!
    integer,dimension(:,:),intent(in)   :: flag
    logical                             :: stop_krig
    logical                             :: edge1,edge2
    integer,intent(inout)               :: nb_samples
    integer,dimension(:,:)              :: points

    edge1 = .false.
    edge2 = .false.
    nb_samples = 0
    allocate(temp(n_max,2))
    temp = 0

    max_lag1 = position(1) - 1
    max_lag2 = position(2) - 1
    max_lag  = min(max_lag1,max_lag2)

    !write(0,*) "max lag z",max_lag1
    !write(0,*) "max lag x",max_lag2
    !write(0,*) "min(max lag)",max_lag

    ! checking if the point to estimate is located near an edge
    if (range .gt. position(2)-1) then 
      edge2 = .true.
    end if
    
    if (range .gt. position(1)-1) then 
      edge1 = .true.
    end if

    ! if the point is located near an edge, setting stop_krig to false, pick as many points as possible even though it might be less than the minimum amount "required"

    if ((edge1 .eqv. .true.) .or. (edge2 .eqv. .true.)) then
      stop_krig = .false.

      !write(0,*) "we are at an edge for quadrant 1"

      do radius=1,max_lag 
        if (nb_samples .eq. n_max) exit
        do lag1 = radius,0,-1
          if (nb_samples .eq. n_max) exit
          if (flag(position(1)-lag1,position(2)-radius) .eq. 1) then 
            nb_samples = nb_samples + 1 
            temp(nb_samples,1) = position(1)-lag1
            temp(nb_samples,2) = position(2)-radius
          end if
        end do

        do lag2 = radius-1,1,-1
          if (nb_samples .eq. n_max) exit
          if (flag(position(1)-radius,position(2)-lag2) .eq. 1) then
            nb_samples = nb_samples + 1
            temp(nb_samples,1) = position(1)-radius
            temp(nb_samples,2) = position(2)-lag2     
          end if
        end do
      end do

      if (max_lag1 .gt. max_lag2) then
        do radius = max_lag+1,max_lag1
          if (nb_samples .eq. n_max) exit
          do lag2 = max_lag,1,-1
            if (nb_samples .eq. n_max) exit
            if (flag(position(1)-radius,position(2)-lag2) .eq. 1) then 
              nb_samples = nb_samples + 1 
              temp(nb_samples,1) = position(1)-radius
              temp(nb_samples,2) = position(2)-lag2
            end if
          end do
        end do
            
      else if (max_lag2 .gt. max_lag1) then

        do radius = max_lag+1,max_lag2
          if (nb_samples .eq. n_max) exit
          do lag1 = max_lag,0,-1
            if (nb_samples .eq. n_max) exit
            if (flag(position(1)-lag1,position(2)-radius) .eq. 1) then 
              nb_samples = nb_samples + 1 
              temp(nb_samples,1) = position(1)-lag1
              temp(nb_samples,2) = position(2)-radius
            end if
          end do
        end do

      end if
      points = temp
               
    ! case where the search area defined by the range does not go off limit
    else 
        do radius = 1,range
          if (nb_samples .eq. n_max) exit
          do lag1 = radius,0,-1
            if (nb_samples .eq. n_max) exit
            if (flag(position(1)-lag1,position(2)-radius) .eq. 1) then 
              nb_samples = nb_samples + 1 
              temp(nb_samples,1) = position(1)-lag1
              temp(nb_samples,2) = position(2)-radius
            end if
          end do
          do lag2 = radius-1,1,-1
            if (nb_samples .eq. n_max) exit
            if (flag(position(1)-radius,position(2)-lag2) .eq. 1) then
              nb_samples = nb_samples + 1
              temp(nb_samples,1) = position(1)-radius
              temp(nb_samples,2) = position(2)-lag2     
            end if
          end do
        end do
        points = temp
        if (nb_samples .ge. n_min) then
          stop_krig = .false. 
        else 
          stop_krig = .true.
        end if

    end if
          
  end subroutine

  subroutine scan2d_2(range,position,n_min,n_max,flag,stop_krig,points,nb_samples) !scanning top right corner
    integer                              :: range,n_min,n_max,lag1,lag2,i,j,max_lag1,max_lag2,radius,nx,nz,max_lag
    integer,allocatable,dimension(:,:)   :: temp
    integer,dimension(2)                 :: position
    integer,dimension(:,:)               :: flag
    logical                              :: stop_krig
    logical                              :: edge1,edge2
    integer,intent(inout)                :: nb_samples
    integer,dimension(:,:)               :: points

    edge1 = .false.
    edge2 = .false.
    nb_samples = 0
    allocate(temp(n_max,2))
    temp = 0
    nz = size(flag,1)
    nx = size(flag,2)

    max_lag1 = position(1) - 1
    max_lag2 = nx - position(2)
    max_lag  = min(max_lag1,max_lag2)

    ! checking if the point to estimate is located near an edge
    if (range .gt. nx-position(2)) then     
      edge2 = .true.
    end if
    
    if (range .gt. position(1)-1) then      
      edge1 = .true.
    end if

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! OLD CODE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 

    ! if the point is located near an edge, set stop_krig to false

!    if ((edge1 .eqv. .true.) .or. (edge2 .eqv. .true.)) then 
!      stop_krig = .false.
!      do i=1,max_lag1
!        if (nb_samples .eq. n_max) exit
!        do j=0,max_lag2
!          if (nb_samples .eq. n_max) exit
!          if (flag(position(1)-i,position(2)+j) .eq. 1) then
!            nb_samples = nb_samples + 1
!            temp(nb_samples,1) = position(1) - i
!            temp(nb_samples,2) = position(2) + j
!          end if
!        end do
!      end do
!      points = temp

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    if ((edge1 .eqv. .true.) .or. (edge2 .eqv. .true.)) then
      stop_krig = .false.

      !write(0,*) "we are at an edge for quadrant 2"

      do radius=1,max_lag 

        if (nb_samples .eq. n_max) exit
        do lag1 = radius,0,-1
          if (nb_samples .eq. n_max) exit
          if (flag(position(1)-radius,position(2)+lag1) .eq. 1) then 
            nb_samples = nb_samples + 1 
            temp(nb_samples,1) = position(1) - radius
            temp(nb_samples,2) = position(2) + lag1
          end if
        end do
        do lag2 = radius-1,1,-1
          if (nb_samples .eq. n_max) exit
          if (flag(position(1)-lag2,position(2)+radius) .eq. 1) then
            nb_samples = nb_samples + 1
            temp(nb_samples,1) = position(1) - lag2
            temp(nb_samples,2) = position(2) + radius     
          end if
        end do
      end do

      if (max_lag1 .gt. max_lag2) then
        do radius = max_lag+1,max_lag1
          if (nb_samples .eq. n_max) exit
          do lag2 = max_lag,0,-1
            if (nb_samples .eq. n_max) exit
            if (flag(position(1)-radius,position(2)+lag2) .eq. 1) then 
              nb_samples = nb_samples + 1 
              temp(nb_samples,1) = position(1) - radius
              temp(nb_samples,2) = position(2) + lag2
            end if
          end do
        end do
            
      else if (max_lag2 .gt. max_lag1) then
        do radius = max_lag+1,max_lag2
          if (nb_samples .eq. n_max) exit
          do lag1 = max_lag,1,-1
            if (nb_samples .eq. n_max) exit
            if (flag(position(1)-lag1,position(2)+radius) .eq. 1) then 
              nb_samples = nb_samples + 1 
              temp(nb_samples,1) = position(1) - lag1
              temp(nb_samples,2) = position(2) + radius
            end if
          end do
        end do

      end if
      points = temp
        
    ! case where the search area defined by the range does not go off limit
    else 
        do radius = 1,range
          if (nb_samples .eq. n_max) exit
          do lag1 = radius,0,-1
            if (nb_samples .eq. n_max) exit
            if (flag(position(1)-radius,position(2)+lag1) .eq. 1) then 
              nb_samples = nb_samples + 1 
              temp(nb_samples,1) = position(1) - radius
              temp(nb_samples,2) = position(2) + lag1
            end if
          end do
          do lag2 = radius-1,1,-1
            if (nb_samples .eq. n_max) exit
            if (flag(position(1)-lag2,position(2)+radius) .eq. 1) then
              nb_samples = nb_samples + 1
              temp(nb_samples,1) = position(1) - lag2
              temp(nb_samples,2) = position(2) + radius     
            end if
          end do
        end do
        points = temp
        if (nb_samples .ge. n_min) then
          stop_krig = .false. 
        else 
          stop_krig = .true.
        end if
    end if          
  end subroutine

  subroutine scan2d_3(range,position,n_min,n_max,flag,stop_krig,points,nb_samples) !scanning bottom right corner
    integer                              :: range,n_min,n_max,nb_samples,lag1,lag2,i,j,max_lag1,max_lag2,radius,nx,nz,max_lag
    integer,allocatable,dimension(:,:)   :: temp
    integer,dimension(2)                 :: position
    integer,dimension(:,:)               :: flag
    logical                              :: stop_krig
    logical                              :: edge1,edge2
    integer,dimension(:,:)               :: points

    edge1 = .false.
    edge2 = .false.
    nb_samples = 0
    allocate(temp(n_max,2))
    temp = 0
    nz = size(flag,1)
    nx = size(flag,2)

    max_lag1 = nz - position(1)
    max_lag2 = nx - position(2)
    max_lag  = min(max_lag1,max_lag2)

    ! checking if the point to estimate is located near an edge
    if (range .gt. nx-position(2)) then       
      edge2 = .true.
    end if
    
    if (range .gt. nz-position(1)) then       
      edge1 = .true.
    end if

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! OLD CODE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  
!   ! if the point is located near an edge, set stop_krig to false
!
!    if ((edge1 .eqv. .true.) .or. (edge2 .eqv. .true.)) then
!      stop_krig = .false. 
!
!
!      do i=0,max_lag1
!        if (nb_samples .eq. n_max) exit
!        do j=1,max_lag2
!          if (nb_samples .eq. n_max) exit
!          if (flag(position(1)+i,position(2)+j) .eq. 1) then
!            nb_samples = nb_samples + 1
!            temp(nb_samples,1) = position(1) + i
!            temp(nb_samples,2) = position(2) + j
!          end if
!        end do
!      end do
!      points = temp
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    if ((edge1 .eqv. .true.) .or. (edge2 .eqv. .true.)) then
      stop_krig = .false.

      !write(0,*) "we are at an edge for quadrant 3"

      do radius=1,max_lag 
        if (nb_samples .eq. n_max) exit
        do lag1 = radius,0,-1
          if (nb_samples .eq. n_max) exit
          if (flag(position(1)+lag1,position(2)+radius) .eq. 1) then 
            nb_samples = nb_samples + 1 
            temp(nb_samples,1) = position(1) + lag1
            temp(nb_samples,2) = position(2) + radius
          end if
        end do

        do lag2 = radius-1,1,-1
          if (nb_samples .eq. n_max) exit
          if (flag(position(1)+radius,position(2)+lag2) .eq. 1) then
            nb_samples = nb_samples + 1
            temp(nb_samples,1) = position(1) + radius
            temp(nb_samples,2) = position(2) + lag2     
          end if
        end do
      end do

      if (max_lag1 .gt. max_lag2) then
        do radius = max_lag+1,max_lag1
          if (nb_samples .eq. n_max) exit
          do lag2 = max_lag,1,-1
            if (nb_samples .eq. n_max) exit
            if (flag(position(1)+radius,position(2)+lag2) .eq. 1) then 
              nb_samples = nb_samples + 1 
              temp(nb_samples,1) = position(1) + radius
              temp(nb_samples,2) = position(2) + lag2
            end if
          end do
        end do
            
      else if (max_lag2 .gt. max_lag1) then
        do radius = max_lag+1,max_lag2
          if (nb_samples .eq. n_max) exit
          do lag1 = max_lag,0,-1
            if (nb_samples .eq. n_max) exit
            if (flag(position(1)+lag1,position(2)+radius) .eq. 1) then 
              nb_samples = nb_samples + 1 
              temp(nb_samples,1) = position(1) + lag1
              temp(nb_samples,2) = position(2) + radius
            end if
          end do
        end do

      end if
      points = temp        

    ! case where the search area defined by the range does not go off limit
    else 
        do radius = 1,range
          if (nb_samples .eq. n_max) exit
          do lag1 = radius,0,-1
            if (nb_samples .eq. n_max) exit
            if (flag(position(1)+lag1,position(2)+radius) .eq. 1) then 
              nb_samples = nb_samples + 1 
              temp(nb_samples,1) = position(1) + lag1
              temp(nb_samples,2) = position(2) + radius
            end if
          end do
          do lag2 = radius-1,1,-1
            if (nb_samples .eq. n_max) exit
            if (flag(position(1)+radius,position(2)+lag2) .eq. 1) then
              nb_samples = nb_samples + 1
              temp(nb_samples,1) = position(1) + radius
              temp(nb_samples,2) = position(2) + lag2     
            end if
          end do
        end do
        points = temp
        if (nb_samples .ge. n_min) then
          stop_krig = .false. 
        else 
          stop_krig = .true.
        end if
    end if          
  end subroutine

  subroutine scan2d_4(range,position,n_min,n_max,flag,stop_krig,points,nb_samples) !scanning bottom left corner
    integer                            :: range,n_min,n_max,nb_samples,lag1,lag2,i,j,max_lag1,max_lag2,radius,nx,nz,max_lag
    integer,allocatable,dimension(:,:) :: temp
    integer,dimension(2)               :: position
    integer,dimension(:,:)             :: flag
    logical                            :: stop_krig
    logical                            :: edge1,edge2
    integer,dimension(:,:)             :: points

    edge1 = .false.
    edge2 = .false.
    nb_samples = 0
    allocate(temp(n_max,2))
    temp = 0
    nz = size(flag,1)
    nx = size(flag,2)

    max_lag1 = nz - position(1)
    max_lag2 = position(2) - 1
    max_lag  = min(max_lag1,max_lag2)

    ! checking if the point to estimate is located near an edge
    if (range .gt. position(2)-1) then 
      edge2 = .true.
    end if
    
    if (range .gt. nz-position(1)) then       
      edge1 = .true.
    end if


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! OLD CODE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
!    ! if the point is located near an edge, set stop_krig to false
!
!    if ((edge1 .eqv. .true.) .or. (edge2 .eqv. .true.)) then
!      stop_krig = .false.
!      do i=1,max_lag1
!        if (nb_samples .eq. n_max) exit
!        do j=0,max_lag2
!          if (nb_samples .eq. n_max) exit
!          if (flag(position(1)+i,position(2)-j) .eq. 1) then
!            nb_samples = nb_samples + 1
!            temp(nb_samples,1) = position(1) + i
!            temp(nb_samples,2) = position(2) - j
!          end if
!        end do
!      end do
!      points = temp
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    if ((edge1 .eqv. .true.) .or. (edge2 .eqv. .true.)) then
      stop_krig = .false.

      !write(0,*) "we are at an edge for quadrant 4"

      do radius=1,max_lag 
        if (nb_samples .eq. n_max) exit
        do lag1 = radius,0,-1
          if (nb_samples .eq. n_max) exit
          if (flag(position(1)+radius,position(2)-lag1) .eq. 1) then 
            nb_samples = nb_samples + 1 
            temp(nb_samples,1) = position(1) + radius
            temp(nb_samples,2) = position(2) - lag1
          end if
        end do

        do lag2 = radius-1,1,-1
          if (nb_samples .eq. n_max) exit
          if (flag(position(1)+lag2,position(2)-radius) .eq. 1) then
            nb_samples = nb_samples + 1
            temp(nb_samples,1) = position(1) + lag2
            temp(nb_samples,2) = position(2) - radius     
          end if
        end do
      end do

      if (max_lag1 .gt. max_lag2) then
        do radius = max_lag+1,max_lag1
          if (nb_samples .eq. n_max) exit
          do lag2 = max_lag,0,-1
            if (nb_samples .eq. n_max) exit
            if (flag(position(1)+radius,position(2)-lag2) .eq. 1) then 
              nb_samples = nb_samples + 1 
              temp(nb_samples,1) = position(1) + radius
              temp(nb_samples,2) = position(2) - lag2
            end if
          end do
        end do
            
      else if (max_lag2 .gt. max_lag1) then
        do radius = max_lag+1,max_lag2
          if (nb_samples .eq. n_max) exit
          do lag1 = max_lag,1,-1
            if (nb_samples .eq. n_max) exit
            if (flag(position(1)+lag1,position(2)-radius) .eq. 1) then 
              nb_samples = nb_samples + 1 
              temp(nb_samples,1) = position(1) + lag1
              temp(nb_samples,2) = position(2) - radius
            end if
          end do
        end do

      end if
      points = temp         

    ! case where the search area defined by the range does not go off limit
    else 
        do radius = 1,range
          if (nb_samples .eq. n_max) exit
          do lag1 = radius,0,-1
            if (nb_samples .eq. n_max) exit
            if (flag(position(1)+radius,position(2)-lag1) .eq. 1) then 
              nb_samples = nb_samples + 1 
              temp(nb_samples,1) = position(1) + radius
              temp(nb_samples,2) = position(2) - lag1
            end if
          end do
          do lag2 = radius-1,1,-1
            if (nb_samples .eq. n_max) exit
            if (flag(position(1)+lag2,position(2)-radius) .eq. 1) then
              nb_samples = nb_samples + 1
              temp(nb_samples,1) = position(1) + lag2
              temp(nb_samples,2) = position(2) - radius     
            end if
          end do
        end do
        points = temp
        if (nb_samples .ge. n_min) then
          stop_krig = .false. 
        else 
          stop_krig = .true.
        end if
    end if          
  end subroutine

  subroutine scan1d_up(range,position,n_min,n_max,flag,stop_krig,points,nb_samples) !scans 1d array from lower to higher index
    integer                           :: n,n_min,n_max,range,position,reach,max_lag,nb_samples,i
    logical                           :: edge,stop_krig
    integer,dimension(:)              :: flag
    real,dimension(:),allocatable     :: temp
    real,dimension(:)                 :: points 
   
    edge = .false.
    nb_samples = 0
    allocate(temp(n_max))
    temp = 0
    n = size(flag)
    reach = position + range

    if (reach .gt. n) then
      edge = .true.
      max_lag = n - position
    end if 

    if (edge .eqv. .true.) then
      do i=1,max_lag
        if (nb_samples .eq. n_max) exit
        if (flag(position+i) .eq. 1) then
          nb_samples = nb_samples + 1
          temp(nb_samples) = position + i
        end if
      end do
      points = temp

    else 
      do i=1,range
        if (nb_samples .eq. n_max) exit
        if (flag(position+i) .eq. 1) then
          nb_samples = nb_samples + 1
          temp(nb_samples) = position + i
        end if
      end do
      points = temp
      if (nb_samples .ge. n_min) then
        stop_krig = .false. 
      else 
        stop_krig = .true.
      end if
    end if

  end subroutine

  subroutine scan1d_down(range,position,n_min,n_max,flag,stop_krig,points,nb_samples) !scans left/right on a 1d dataset
    integer                           :: n,n_min,n_max,range,position,reach,max_lag,i,nb_samples
    logical                           :: edge,stop_krig
    integer,dimension(:)              :: flag
    real,dimension(:),allocatable     :: temp
    real,dimension(:)                 :: points 
    
    edge = .false.
    nb_samples = 0
    allocate(temp(n_max))
    temp = 0
    n = size(flag)
    reach = position - range

    if (reach .lt. 1) then
      edge = .true.
      max_lag = position - 1
    end if 

    if (edge .eqv. .true.) then
      do i=1,max_lag
        if (nb_samples .eq. 1) exit
        if (flag(position-i) .eq. 1) then
          nb_samples = nb_samples + 1
          temp(nb_samples) = position - i
        end if
      end do
      points = temp

    else 
      do i=1,range
        if (nb_samples .eq. n_max) exit
        if (flag(position-i) .eq. 1) then
          nb_samples = nb_samples + 1
          temp(nb_samples) = position - i
        end if
      end do
      points = temp
      if (nb_samples .ge. n_min) then
        stop_krig = .false. 
      else 
        stop_krig = .true.
      end if
    end if
  end subroutine

end module
