module syncmp_mod

  implicit none 
  contains

  subroutine syncmp1(data, nt, nh, ny, nz, maxamp, maxdelay)
    integer         :: nt, nh, ny, nz, ns, ng
    integer         :: it, ih, iy, is, iz, iseed, ig
    real            :: data(nt, nh, ny), refl(nz, ny), depth(nz), shotamp(ny)
    real            :: maxamp, maxdelay, layer, rand01
    integer         :: geodelay(ny)

    iseed = 1992
    ns = ny
    ng = ny

    do iz=1,nz
      ! 0 < rand01() < 1
      depth( iz) = nt*rand01(iseed) ! Reflector depth
      layer = 2.*rand01(iseed) - 1. ! Reflector strength
      do iy=1,ny
        ! Impose texture on layer
        refl(iz, iy) = layer*(1.+rand01(iseed))
      enddo
    enddo

    do is=1,ns
      shotamp(is) = 1. + (maxamp-1.)*rand01(iseed)
    enddo
    do ig=1,ng
      geodelay(ig) = maxdelay*rand01(iseed)
    enddo

    data = 0.

    do iy=1,ny ! midpoint
      do ih=1,nh ! down cable h = (g-s)/2
        is = iy - (ih-1) ! s = shotpoint
        if(is <= 0) then
          is = ny + is
        endif
        ! periodic with midpoint
        ig = iy + (ih-1) ! g = geophone location
        ig = 1 + (ig-ng*(ig/ng)) ! periodic with midpoint

        do iz=1,nz
          ! Add hyperbola for each layer
          it = 1 + sqrt(depth(iz)**2 + 25.*(ih-1)**2) + geodelay(ig)
          if(it <= nt) then
            data(it, ih, iy) = data(it, ih, iy) + refl(iz, iy)*shotamp(is)
          endif
        enddo
      enddo
    enddo

  end subroutine

end module
