program test_kriging
  
  use sep
  use scan_mod
  use kriging_mod

  implicit none

  integer                              :: nz,nx,n,range,n_min,n_max,nb1,nb2,nb3,nb4,known_samp,nb_samp_krig
  real                                 :: oz,dz,ox,dx,z_coord,x_coord
  real,dimension(:,:),allocatable      :: sparse_data,temp
  integer,dimension(:,:),allocatable   :: flag,flag_krig,flag_mc,flag_start
  real,dimension(:,:),allocatable      :: rflag,rflag_krig,rflag_mc,rflag_start,rflag_sum
  real                                 :: rand_nb
  logical                              :: stop_krig
  integer,dimension(2)                 :: position
  integer                              :: i,j,k,by_krig,by_mc,beginning,at_end,m1,m2
  real                                 :: nugget,range_var,sill,alpha
  real,dimension(:),allocatable        :: samp_values
  integer,dimension(:,:),allocatable   :: samp_position,samp1,samp2,samp3,samp4
  
  call sep_init()

  call from_history("n1", nz) 
  call from_history("o1", oz)
  call from_history("d1", dz)
  call from_history("n2", nx) 
  call from_history("o2", ox)
  call from_history("d2", dx)

  call from_param("n_max",n_max) 
  call from_param("n_min",n_min)
  call from_param("range",range)

  call from_param("nugget",nugget)
  call from_param("range_variogram",range_var)
  call from_param("sill",sill)
  call from_param("alpha",alpha)

  call from_param("m1",m1)
  call from_param("m2",m2)

  allocate(sparse_data(nz,nx))
  allocate(flag(nz,nx))
  allocate(temp(nz,nx))

  allocate(samp1(n_max,2))
  allocate(samp2(n_max,2))
  allocate(samp3(n_max,2))
  allocate(samp4(n_max,2))

  call sep_read(sparse_data)
  call sep_read(temp,"flag") ! flag_matrix.H is an auxiliary file

  call ordinary_kriging_init(nugget,range_var,sill,alpha)

  flag = temp ! flag has to be of type integer, whereas temp is the array read using sep_read (so we need to convert it).
  deallocate(temp)

  allocate(flag_krig(nz,nx)) !flag_krig=1 when the value has been determined using kriging
  allocate(flag_mc(nz,nx)) ! flag_mc = 1 when the value has been generated usign MC
  allocate(flag_start(nz,nx)) ! flag_start=1 when the value was known from the start

  allocate(rflag(nz,nx)) ! flag=1 when the point has a known value
  allocate(rflag_krig(nz,nx)) !flag_krig=1 when the value has been determined using kriging
  allocate(rflag_mc(nz,nx)) ! flag_mc = 1 when the value has been generated usign MC
  allocate(rflag_start(nz,nx)) ! flag_start=1 when the value was known from the start
  allocate(rflag_sum(nz,nx))

  !!!!!!!!  saving the starting flag array in flag_start !!!!!!!! 
  flag_start = flag 
  flag_krig = 0
  flag_mc = 0
 
  !!!!!!!! getting the original nb of known samples !!!!!!!!
  known_samp = 0
  do i=1,nz
    do j=1,nx
      known_samp = known_samp + flag(i,j)
    end do
  end do

  write(0,*) "known samp", known_samp

  n = nz * nx
 
  i=m1
  j=m2

  samp1=0;samp2=0;samp3=0;samp4=0
  nb1=0;nb2=0;nb3=0;nb4=0

  position(1) = i
  position(2) = j       
  stop_krig= .false.

  call scan2d_1(range,position,n_min,n_max,flag,stop_krig,samp1,nb1) !scan top left
    if (stop_krig .eqv. .false.) then
      call scan2d_2(range,position,n_min,n_max,flag,stop_krig,samp2,nb2) !scan top right
        if (stop_krig .eqv. .false.) then
          call scan2d_3(range,position,n_min,n_max,flag,stop_krig,samp3,nb3) !scan bottom right
            if (stop_krig .eqv. .false.) then
              call scan2d_4(range,position,n_min,n_max,flag,stop_krig,samp4,nb4) !scan bottom left
            end if
        end if
    end if

      write(0,*) "nb1",nb1
      write(0,*) "nb2",nb2
      write(0,*) "nb3",nb3
      write(0,*) "nb4",nb4

    write(0,*) "stop krig", stop_krig
    if (stop_krig .eqv. .false.) then
      nb_samp_krig = nb1 + nb2 + nb3 + nb4
      allocate(samp_position(nb_samp_krig,2))
      allocate(samp_values(nb_samp_krig))
 
      if (nb1 .gt. 0) then
        do k=1,nb1
          write(0,*) "samp1(",k,",1) = ",samp1(k,1)
          write(0,*) "samp1(",k,",2) = ",samp1(k,2)
          write(0,*) "val1(",k,",2) = ",sparse_data(samp1(k,1),samp1(k,2))
        end do
        do k=1,nb1
          samp_position(k,1) = samp1(k,1)
          samp_position(k,2) = samp1(k,2)
          samp_values(k)     = sparse_data(samp1(k,1),samp1(k,2))
        end do
      end if
      if (nb2 .gt. 0) then
        do k=1,nb2
          write(0,*) "samp2(",k,",1) = ",samp2(k,1)
          write(0,*) "samp2(",k,",2) = ",samp2(k,2)
          write(0,*) "val2(",k,",2) = ",sparse_data(samp2(k,1),samp2(k,2))
        end do
        do k=1,nb2
              samp_position(k+nb1,1) = samp2(k,1)
              samp_position(k+nb1,2) = samp2(k,2)
              samp_values(k+nb1)     = sparse_data(samp2(k,1),samp2(k,2))
        end do
      end if
      if (nb3 .gt. 0) then
        do k=1,nb3
          write(0,*) "samp3(",k,",1) = ",samp3(k,1)
          write(0,*) "samp3(",k,",2) = ",samp3(k,2)
          write(0,*) "val3(",k,",2) = ",sparse_data(samp3(k,1),samp3(k,2))
        end do
        do k=1,nb3
          samp_position(k+nb1+nb2,1) = samp3(k,1)
          samp_position(k+nb1+nb2,2) = samp3(k,2)
          samp_values(k+nb1+nb2)     = sparse_data(samp3(k,1),samp3(k,2))
        end do
      end if
      if (nb4 .gt. 0) then
        do k=1,nb4
          write(0,*) "samp4(",k,",1) = ",samp4(k,1)
          write(0,*) "samp4(",k,",2) = ",samp4(k,2)
          write(0,*) "val4(",k,",2) = ",sparse_data(samp4(k,1),samp4(k,2))
        end do
        do k=1,nb4
          samp_position(k+nb1+nb2+nb3,1) = samp4(k,1)
          samp_position(k+nb1+nb2+nb3,2) = samp4(k,2)
          samp_values(k+nb1+nb2+nb3)     = sparse_data(samp4(k,1),samp4(k,2))
        end do
      end if

      !!!!!!!! calling the kriging function for the unknown point  

      sparse_data(i,j) = ordinary_kriging(samp_position,samp_values,position)
      write(0,*) "kriged value",sparse_data(i,j)
      flag_krig(i,j) = 1
      deallocate(samp_position)
      deallocate(samp_values)

    else
      !sparse_data(i,j) = generate_random_val(cdf)
      sparse_data(i,j) = -10.0
      flag_mc(i,j) = 1
    end if

    flag(i,j) = 1
    known_samp = known_samp + 1
    write(0,*) "new nb", known_samp

  call sep_close()

end program
